/******************************************************************************
 Web Notification Service (WNS): An open interface for a service
 by which a client may conduct asynchronous dialogues
 (message interchanges) with one or more other services.
 ******************************************************************************/
package org.n52.wns;

import java.io.IOException;
import java.util.Enumeration;
import java.util.logging.Level;
import java.util.logging.Logger;

import javax.servlet.ServletConfig;
import javax.servlet.ServletContext;
import javax.servlet.ServletException;
import javax.servlet.http.HttpServlet;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

/**
 * @author Johannes Echterhoff, Dennis Dahlmann
 * @version 1.0
 */
public class WNS extends HttpServlet {

	private Logger log = Logger.getLogger(WNS.class.getName());

	private WNSInitParamContainer initParams = null;

	/**
	 * Init method to check the config parameters
	 */
	public void init(ServletConfig config) throws ServletException {
		super.init(config);

		// --- Ausgabe der Ini-Parameter zur Kontrolle
		ServletContext context = getServletContext();
		Enumeration enumer = context.getInitParameterNames();

		while (enumer.hasMoreElements()) {
			String key = (String) enumer.nextElement();
			Object value = context.getInitParameter(key);
			log.info("Context_Init_Parameter: " + key + " = " + value);
			System.out
					.println("Context_Init_Parameter: " + key + " = " + value);
		}

		String wnsConfigLocation = context
				.getInitParameter("WNS_CONFIG_FILE_LOCATION");

		try {
			this.initParams = WNSInitParamContainer
					.createInstance(wnsConfigLocation);
		} catch (WNSException e) {
			throw new ServletException("Could not initialise WNS.", e);
		}
	}

	/**
	 * Returns the Servlet info
	 */
	public String getServletInfo() {
		return "Web Notification Service";
	}

	/**
	 * @see javax.servlet.http.HttpServlet#doGet(javax.servlet.http.HttpServletRequest,
	 *      javax.servlet.http.HttpServletResponse)
	 */
	public void doGet(HttpServletRequest request, HttpServletResponse response)
			throws ServletException, IOException {

		try {
			WNSRequestHandler requestHandler = new WNSRequestHandler();
			requestHandler.handleGetRequest(request, response);

		} catch (Exception e) {

			log.log(Level.WARNING,
					"An exception occurred while handling Get request.", e);

			// try to send ExceptionReport
			WNSServiceException se = new WNSServiceException(initParams
					.getExceptionLevel());
			se
					.addCodedException(
							WNSServiceException.ExceptionCode.NoApplicableCode,
							null, e);
			try {
				WNSUtil.sendResponse(se.getDocument(), initParams
						.getOgcExceptionSchemaLocation(), response);
			} catch (WNSException e1) {
				// log that the exception could not be sent
				log.log(Level.WARNING, "The exception could not be sent.", e);
			}
		}
	}

	/**
	 * @see javax.servlet.http.HttpServlet#doGet(javax.servlet.http.HttpServletRequest,
	 *      javax.servlet.http.HttpServletResponse)
	 */
	public void doPost(HttpServletRequest request, HttpServletResponse response)
			throws ServletException, IOException {

		try {
			WNSRequestHandler requestHandler = new WNSRequestHandler();
			requestHandler.handlePostRequest(request, response);

		} catch (Exception e) {

			log.log(Level.WARNING,
					"An exception occurred while handling Post request.", e);

			// try to send ExceptionReport
			WNSServiceException se = new WNSServiceException(initParams
					.getExceptionLevel());
			se
					.addCodedException(
							WNSServiceException.ExceptionCode.NoApplicableCode,
							null, e);
			try {
				WNSUtil.sendResponse(se.getDocument(), initParams
						.getOgcExceptionSchemaLocation(), response);
			} catch (WNSException e1) {
				// log that the exception could not be sent
				log.log(Level.WARNING, "The exception could not be sent.", e);
			}
		}
	}
}