/******************************************************************************
 Web Notification Service (WNS): An open interface for a service
 by which a client may conduct asynchronous dialogues
 (message interchanges) with one or more other services.
 *******************************************************************************/
package org.n52.wns;

/**
 * This class provides the necassary methods to add and delete entries from the
 * database after a specific duration
 * 
 * @author Dennis Dahlmann
 * @version 1.0
 */

import java.text.SimpleDateFormat;
import java.util.Calendar;
import java.util.GregorianCalendar;
import java.util.Hashtable;
import java.util.Vector;
import java.util.logging.Logger;

import noNamespace.XMLHashTableDocument;

import org.apache.xmlbeans.GDuration;
import org.apache.xmlbeans.XmlException;
import org.xmldb.api.base.Collection;
import org.xmldb.api.base.XMLDBException;
import org.xmldb.api.modules.XMLResource;

public class WNSDatabaseThread extends Thread implements Runnable {

	private int updateRate;

	private Collection col;

	private Vector<Long> list = new Vector<Long>();

	private XMLResource r;

	// ExpiredDate : MessageID
	private Hashtable<Long, String> hashTable = new Hashtable<Long, String>();

	private XMLHashTableDocument xmlhash = XMLHashTableDocument.Factory
			.newInstance();

	private SimpleDateFormat format = new SimpleDateFormat("yyyyMMddHHmmss");

	private static Logger logger = Logger.getLogger(WNSDatabaseThread.class
			.getName());

	/**
	 * 
	 * @param updateRate
	 *            The time in minutes to check for old database entries
	 * @param col
	 *            The name of the Collection to search in
	 */
	public WNSDatabaseThread(int updateRate, Collection col) {
		super();
		xmlhash.addNewXMLHashTable();
		this.updateRate = updateRate * 60000;
		this.col = col;
		this.init();
		start();

	}

	private void init() {
		try {
			r = (XMLResource) col.getResource("savedIDs");
			if (r != null) {
				xmlhash = XMLHashTableDocument.Factory.parse(r
						.getContentAsDOM());
				for (int i = 0; i < xmlhash.getXMLHashTable().sizeOfHashArray(); i++) {
					hashTable.put(xmlhash.getXMLHashTable().getHashArray(i)
							.getDate(), xmlhash.getXMLHashTable().getHashArray(
							i).getMessageID());
					list.addElement(xmlhash.getXMLHashTable().getHashArray(i)
							.getDate());
				}
			}
			col.close();
		} catch (XMLDBException e) {

		} catch (XmlException e) {

		}

	}

	/**
	 * Run method to start the thread
	 */
	public void run() {
		while (true) {
			try {
				Thread.sleep(updateRate);
				if (list.size() > 0) {
					checkExpiredIDs();
				}
			} catch (InterruptedException e) {

			}
		}
	}

	private void checkExpiredIDs() {

		GregorianCalendar cal = new GregorianCalendar();
		Long time = Long.parseLong(format.format(cal.getTime()));
		if (list.size() > 0) {
			Vector<Integer> delete = new Vector<Integer>();
			for (int i = 0; i < list.size(); i++) {
				if (list.get(i) <= time) {
					delete.addElement(i);
					logger.info("MessageID: " + hashTable.get(list.get(i))
							+ " expired");
					try {
						r = (XMLResource) col.createResource((String) hashTable
								.get(list.get(i)), "XMLResource");
						col.removeResource(r);
						col.close();
					} catch (XMLDBException e) {

					}
					hashTable.remove(list.get(i));
					for (int j = 0; j < xmlhash.getXMLHashTable()
							.sizeOfHashArray(); j++) {
						if (xmlhash.getXMLHashTable().getHashArray(j).getDate() == list
								.get(i)) {
							xmlhash.getXMLHashTable().removeHash(j);
							XMLResource document;
							try {
								document = (XMLResource) col.createResource(
										"savedIDs", "XMLResource");
								document.setContent(xmlhash);
								col.storeResource(document);
								col.close();
							} catch (XMLDBException e) {
								// TODO Auto-generated catch block
								e.printStackTrace();
							}
						}
					}

				}
			}
			for (Integer integer : delete) {
				list.removeElementAt(integer);
			}

			list.trimToSize();
		}

	}

	/**
	 * 
	 * @param id
	 *            The ID of the message
	 * @param duration
	 *            The duration the message will be in the database
	 */
	public void add(String id, GDuration duration) {
		GregorianCalendar cal = new GregorianCalendar();
		cal.add(Calendar.YEAR, duration.getYear());
		cal.add(Calendar.MONTH, duration.getMonth());
		cal.add(Calendar.DATE, duration.getDay());
		cal.add(Calendar.HOUR, duration.getHour());
		cal.add(Calendar.MINUTE, duration.getMinute());
		cal.add(Calendar.SECOND, duration.getSecond());

		Long time = Long.parseLong(format.format(cal.getTime()));
		hashTable.put(time, id);
		noNamespace.XMLHashTableDocument.XMLHashTable.Hash hash = xmlhash
				.getXMLHashTable().addNewHash();
		hash.setDate(time);
		hash.setMessageID(id);
		list.addElement(time);
		XMLResource document;
		try {
			document = (XMLResource) col.createResource("savedIDs",
					"XMLResource");
			document.setContent(xmlhash);
			col.storeResource(document);
			col.close();
		} catch (XMLDBException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}

	}

}
