/******************************************************************************
 Web Notification Service (WNS): An open interface for a service
 by which a client may conduct asynchronous dialogues
 (message interchanges) with one or more other services.
 *******************************************************************************/

package org.n52.wns;

/**
 * @author Dennis Dahlmann
 * @version 1.0
 */

public class WNSException extends Exception {

	public final static String NESTED_EXCEPTION = "Nested Exception";

	// locator and exception
	private String locator;

	private Exception nested;

	/** constructor just calls super constructor */
	public WNSException() {
		super();
	}

	/**
	 * constructor
	 * 
	 * @param message
	 *            String representation of the exception message
	 */
	public WNSException(String message) {
		super(message);
	}

	/**
	 * constructor
	 * 
	 * @param message
	 *            String representation of the exception message
	 * @param locator
	 *            String representation of the locator
	 */
	public WNSException(String message, String locator) {
		this(message);
		this.locator = locator;
	}

	/**
	 * constructor
	 * 
	 * @param nested
	 *            Exception
	 */
	public WNSException(Exception nested) {
		this(NESTED_EXCEPTION, nested);
	}

	/**
	 * constructor
	 * 
	 * @param message
	 *            String representation of the exception message
	 * @param nested
	 *            exception
	 */
	public WNSException(String message, Exception nested) {
		super(message);
		this.nested = nested;
	}

	/**
	 * gets a String representation of the exception message
	 * 
	 * @return String
	 */
	public String getMessage() {
		StringBuffer sb = new StringBuffer();
		if (super.getMessage() != null) {
			sb.append(super.getMessage());
		}
		if (nested != null) {
			sb.append(" >> ");
			sb.append(nested.getMessage());
		}
		return sb.toString();
	}

	/**
	 * gets the locator
	 * 
	 * @return locator
	 */
	public String getLocator() {
		return locator;
	}

	public String getDefinition() {
		return null;
	}

	private String getElementName() {
		return "WNSException";
	}

	/**
	 * private getValue
	 * 
	 * @return String representation
	 */
	private String getValue() {
		StringBuffer sb = new StringBuffer();
		sb.append("<Exception>");
		if (locator != null) {
			sb.append("<Locator>");
			sb.append(locator);
			sb.append("</Locator>");
		}
		sb.append("<Message>");
		if (getMessage() != null) {
			sb.append(getMessage());
		} else {
			sb.append("No detailed message available");
		}
		sb.append("</Message>");
		sb.append("</Exception>");
		return sb.toString();
	}
}
