/******************************************************************************
 Web Notification Service (WNS): An open interface for a service
 by which a client may conduct asynchronous dialogues
 (message interchanges) with one or more other services.
 *******************************************************************************/

package org.n52.wns;

import java.util.Vector;
import java.util.logging.Logger;

import net.opengis.wns.RegisterMultiUserType;
import net.opengis.wns.RegisterSingleUserType;
import net.opengis.wns.UpdateMultiUserRegistrationType.Action.AddUser;
import net.opengis.wns.UpdateMultiUserRegistrationType.Action.RemoveUser;
import net.opengis.wns.UpdateSingleUserRegistrationType.Action.AddCommunicationProtocols;
import net.opengis.wns.UpdateSingleUserRegistrationType.Action.RemoveCommunicationProtocols;

import org.x52North.wns.WNSUserDocument;
import org.x52North.wns.WNSUserDocument.WNSUser.MultiUser.User.AssociatedUsers;
import org.x52North.wns.WNSUserDocument.WNSUser.SingleUser.User.Channel;

/**
 * This class provides the necassary methods to handle user
 * 
 * @author Dennis Dahlmann
 * @version 1.0
 */
public class WNSUser {
	private String name;

	private long userID;

	private String[] multiUserID;

	private boolean multiUser;

	private String[] email;

	private String[] fax;

	private String[] sms;

	private String[] xmpp;

	private String[] phone;

	private String[] http;

	private static Logger log = Logger.getLogger(WNSUser.class.getName());

	/**
	 * 
	 * @param rsut
	 *            The XML document to register a SingleUser, see schema
	 * @param id
	 */
	public WNSUser(RegisterSingleUserType rsut, long id) {
		this.name = rsut.getName();
		this.userID = id;

		if (rsut.getCommunicationProtocol().getEmailArray() != null) {
			this.email = rsut.getCommunicationProtocol().getEmailArray();
		}
		if (rsut.getCommunicationProtocol().getXMPPArray() != null) {
			this.xmpp = rsut.getCommunicationProtocol().getXMPPArray();
		}
		if (rsut.getCommunicationProtocol().getHTTPArray() != null) {
			this.http = rsut.getCommunicationProtocol().getHTTPArray();
		}
		if (rsut.getCommunicationProtocol().getFaxArray() != null) {
			this.fax = new String[rsut.getCommunicationProtocol().getFaxArray().length];
			for (int i = 0; i <= fax.length - 1; i++) {
				fax[i] = rsut.getCommunicationProtocol().getFaxArray(i)
						.toString();
			}

		}
		if (rsut.getCommunicationProtocol().getPhoneArray() != null) {
			this.phone = new String[rsut.getCommunicationProtocol()
					.getPhoneArray().length];
			for (int i = 0; i <= phone.length - 1; i++) {
				phone[i] = rsut.getCommunicationProtocol().getPhoneArray(i)
						.toString();
			}

		}
		if (rsut.getCommunicationProtocol().getSMSArray() != null) {
			this.sms = new String[rsut.getCommunicationProtocol().getSMSArray().length];
			for (int i = 0; i <= sms.length - 1; i++) {
				sms[i] = rsut.getCommunicationProtocol().getSMSArray(i)
						.toString();
			}
		}
	}

	/**
	 * 
	 * @param rmut
	 *            The XML document to register a MultiUser
	 * @param id
	 */
	public WNSUser(RegisterMultiUserType rmut, long id) {
		this.userID = id;
		this.multiUser = true;
		multiUserID = rmut.getUserIDArray();
	}

	public WNSUser(
			org.x52North.wns.WNSUserDocument.WNSUser.SingleUser.User userArray) {
		this.name = userArray.getName();
		this.userID = Long.parseLong(userArray.getID());
		for (int i = 0; i <= userArray.getChannelArray().length - 1; i++) {
			if (userArray.getChannelArray()[i]
					.getProtocol()
					.equals(
							WNSUserDocument.WNSUser.SingleUser.User.Channel.Protocol.EMAIL)) {
				email = userArray.getChannelArray()[i].getTargetArray();
			}
			if (userArray.getChannelArray()[i]
					.getProtocol()
					.equals(
							WNSUserDocument.WNSUser.SingleUser.User.Channel.Protocol.FAX)) {
				fax = userArray.getChannelArray()[i].getTargetArray();
			}
			if (userArray.getChannelArray()[i]
					.getProtocol()
					.equals(
							WNSUserDocument.WNSUser.SingleUser.User.Channel.Protocol.PHONE)) {
				phone = userArray.getChannelArray()[i].getTargetArray();
			}
			if (userArray.getChannelArray()[i]
					.getProtocol()
					.equals(
							WNSUserDocument.WNSUser.SingleUser.User.Channel.Protocol.SMS)) {
				sms = userArray.getChannelArray()[i].getTargetArray();
			}
			if (userArray.getChannelArray()[i]
					.getProtocol()
					.equals(
							WNSUserDocument.WNSUser.SingleUser.User.Channel.Protocol.HTTP)) {
				http = userArray.getChannelArray()[i].getTargetArray();
			}
			if (userArray.getChannelArray()[i]
					.getProtocol()
					.equals(
							WNSUserDocument.WNSUser.SingleUser.User.Channel.Protocol.XMPP)) {
				xmpp = userArray.getChannelArray()[i].getTargetArray();
			}

		}

	}

	public WNSUser(
			org.x52North.wns.WNSUserDocument.WNSUser.MultiUser.User userArray) {
		this.userID = Long.parseLong(userArray.getID());
		this.multiUser = true;
		this.multiUserID = userArray.getAssociatedUsers().getIDArray();
	}

	/**
	 * Returns the name of the user
	 * 
	 * @return name String
	 */
	public String getName() {
		return name;
	}

	/**
	 * Sets the name of the user
	 * 
	 * @param name
	 *            user name
	 */
	public void setName(String name) {
		this.name = name;
	}

	/**
	 * 
	 * @return The associated users as in the schema
	 */
	public AssociatedUsers getAssociatedUsers() {
		if (isMultiUser()) {
			org.x52North.wns.WNSUserDocument.WNSUser.MultiUser.User user = org.x52North.wns.WNSUserDocument.WNSUser.MultiUser.User.Factory
					.newInstance();
			user.addNewAssociatedUsers();
			user.getAssociatedUsers().setIDArray(multiUserID);
			return user.getAssociatedUsers();
		} else {
			return null;
		}
	}

	/**
	 * Returns all available channels from a WNSUser
	 * 
	 * @return The vector which contains all available channels
	 */
	public Channel[] getChannels() {
		org.x52North.wns.WNSUserDocument.WNSUser.SingleUser.User user = org.x52North.wns.WNSUserDocument.WNSUser.SingleUser.User.Factory
				.newInstance();
		if (email != null && email.length > 0) {
			Channel chan = user.addNewChannel();
			chan
					.setProtocol(WNSUserDocument.WNSUser.SingleUser.User.Channel.Protocol.EMAIL);
			chan.setTargetArray(email);
		}
		if (fax != null && fax.length > 0) {
			Channel chan = user.addNewChannel();
			chan
					.setProtocol(WNSUserDocument.WNSUser.SingleUser.User.Channel.Protocol.FAX);
			chan.setTargetArray(fax);
		}
		if (phone != null && phone.length > 0) {
			Channel chan = user.addNewChannel();
			chan
					.setProtocol(WNSUserDocument.WNSUser.SingleUser.User.Channel.Protocol.PHONE);
			chan.setTargetArray(phone);
		}
		if (sms != null && sms.length > 0) {
			Channel chan = user.addNewChannel();
			chan
					.setProtocol(WNSUserDocument.WNSUser.SingleUser.User.Channel.Protocol.SMS);
			chan.setTargetArray(sms);
		}
		if (http != null && http.length > 0) {
			Channel chan = user.addNewChannel();
			chan
					.setProtocol(WNSUserDocument.WNSUser.SingleUser.User.Channel.Protocol.HTTP);
			chan.setTargetArray(http);
		}
		if (xmpp != null && xmpp.length > 0) {
			Channel chan = user.addNewChannel();
			chan
					.setProtocol(WNSUserDocument.WNSUser.SingleUser.User.Channel.Protocol.XMPP);
			chan.setTargetArray(xmpp);
		}
		return user.getChannelArray();
	}

	/**
	 * gets the UserID
	 * 
	 * @return UserID long
	 */
	public long getUserID() {
		return userID;
	}

	/**
	 * sets the UserID
	 * 
	 * @param userid
	 *            long
	 */
	public void setUserID(long userid) {
		this.userID = userid;
	}

	/**
	 * Checks, if the WNSUser is a MultiUser or not
	 * 
	 * @return true, if the user is a MultiUser, false if not
	 */
	public boolean isMultiUser() {
		return multiUser;
	}

	/**
	 * 
	 * @param removeCommunicationProtocols
	 *            The protocols that should be removed
	 */
	public void removeProtocol(
			RemoveCommunicationProtocols removeCommunicationProtocols) {

		if (email != null && email.length > 0
				&& removeCommunicationProtocols.sizeOfEmailArray() > 0) {
			// Only one element?
			if (email.length == 1) {
				email = null;
			} else {
				int emailInt = 0;
				String[] remove = removeCommunicationProtocols.getEmailArray();
				for (int i = 0; i <= remove.length - 1; i++) {
					for (int j = 0; j <= email.length - 1; j++) {
						if (remove[i].equalsIgnoreCase(email[j])) {
							email[j] = null;
							emailInt++;
						}
					}
				}
				String[] temp = new String[email.length - emailInt];
				int index = 0;
				for (int i = 0; i < email.length; i++) {
					if (email[i] != null) {
						temp[index] = email[i];
					}
				}
				email = temp;
			}
		}
		if (fax != null && fax.length > 0
				&& removeCommunicationProtocols.sizeOfFaxArray() > 0) {
			// Only one element?
			if (fax.length == 1) {
				fax = null;

			} else {
				int faxInt = 0;
				String[] remove = new String[removeCommunicationProtocols
						.sizeOfFaxArray()];
				for (int i = 0; i < fax.length; i++) {
					remove[i] = removeCommunicationProtocols.getFaxArray(i)
							.toString();
				}

				for (int i = 0; i <= remove.length - 1; i++) {
					for (int j = 0; j <= fax.length - 1; j++) {
						if (remove[i].equalsIgnoreCase(fax[j])) {
							fax[j] = null;
							faxInt++;
						}
					}
				}
				String[] temp = new String[fax.length - faxInt];
				int index = 0;
				for (int i = 0; i < fax.length; i++) {
					if (fax[i] != null) {
						temp[index] = fax[i];
					}
				}
				fax = temp;
			}
		}
		if (phone != null && phone.length > 0
				&& removeCommunicationProtocols.sizeOfPhoneArray() > 0) {
			// Only one element?
			if (phone.length == 1) {
				phone = null;

			} else {
				int phoneInt = 0;
				String[] remove = new String[removeCommunicationProtocols
						.sizeOfPhoneArray()];
				for (int i = 0; i <= removeCommunicationProtocols
						.getPhoneArray().length - 1; i++) {
					remove[i] = removeCommunicationProtocols.getPhoneArray(i)
							.toString();
				}

				for (int i = 0; i <= remove.length - 1; i++) {
					for (int j = 0; j <= phone.length - 1; j++) {
						if (remove[i].equalsIgnoreCase(phone[j])) {
							phone[j] = null;
							phoneInt++;
						}
					}
				}
				String[] temp = new String[phone.length - phoneInt];
				int index = 0;
				for (int i = 0; i < phone.length; i++) {
					if (phone[i] != null) {
						temp[index] = phone[i];
					}
				}
				phone = temp;
			}
		}
		if (sms != null && sms.length > 0
				&& removeCommunicationProtocols.sizeOfSMSArray() > 0) {
			// Only one element?
			if (sms.length == 1) {
				sms = null;

			} else {
				int smsInt = 0;
				String[] remove = new String[removeCommunicationProtocols
						.sizeOfSMSArray()];
				for (int i = 0; i <= removeCommunicationProtocols.getSMSArray().length - 1; i++) {
					remove[i] = removeCommunicationProtocols.getSMSArray(i)
							.toString();
				}

				for (int i = 0; i <= remove.length - 1; i++) {
					for (int j = 0; j <= sms.length - 1; j++) {
						if (remove[i].equalsIgnoreCase(sms[j])) {
							sms[j] = null;
							smsInt++;
						}
					}
				}
				String[] temp = new String[sms.length - smsInt];
				int index = 0;
				for (int i = 0; i < sms.length; i++) {
					if (sms[i] != null) {
						temp[index] = sms[i];
					}
				}
				sms = temp;
			}
		}
		if (http != null && http.length > 0
				&& removeCommunicationProtocols.sizeOfHTTPArray() > 0) {
			// Only one element?
			if (http.length == 1) {
				http = null;

			} else {
				int httpInt = 0;
				String[] remove = removeCommunicationProtocols.getHTTPArray();
				for (int i = 0; i <= remove.length - 1; i++) {
					for (int j = 0; j <= http.length - 1; j++) {
						if (remove[i].equalsIgnoreCase(http[j])) {
							http[j] = null;
							httpInt++;
						}
					}
				}
				String[] temp = new String[http.length - httpInt];
				int index = 0;
				for (int i = 0; i < http.length; i++) {
					if (http[i] != null) {
						temp[index] = http[i];
					}
				}
				http = temp;
			}
		}
		if (xmpp != null && xmpp.length > 0
				&& removeCommunicationProtocols.sizeOfXMPPArray() > 0) {
			// Only one element?
			if (xmpp.length == 1) {
				xmpp = null;

			} else {
				int xmppInt = 0;
				String[] remove = removeCommunicationProtocols.getXMPPArray();
				for (int i = 0; i <= remove.length - 1; i++) {
					for (int j = 0; j <= xmpp.length - 1; j++) {
						if (remove[i].equalsIgnoreCase(xmpp[j])) {
							xmpp[j] = null;
							xmppInt++;
						}
					}
				}
				String[] temp = new String[xmpp.length - xmppInt];
				int index = 0;
				for (int i = 0; i < xmpp.length; i++) {
					if (xmpp[i] != null) {
						temp[index] = xmpp[i];
					}
				}
				xmpp = temp;
			}
		}

	}

	/**
	 * 
	 * @param addCommunicationProtocols
	 *            The protocols that should be added
	 */
	public void addProtocol(AddCommunicationProtocols addCommunicationProtocols) {

		if (addCommunicationProtocols.sizeOfEmailArray() > 0) {
			if (email != null) {
				String[] add = addCommunicationProtocols.getEmailArray();
				String[] newer = new String[email.length + add.length];
				for (int i = 0; i <= email.length - 1; i++) {
					newer[i] = email[i];
				}
				for (int j = 0; j <= add.length - 1; j++) {
					newer[email.length + j] = add[j];
				}
				email = newer;
			} else {
				email = addCommunicationProtocols.getEmailArray();
			}
		}

		if (addCommunicationProtocols.sizeOfFaxArray() > 0) {
			if (fax != null) {
				String[] temp = new String[addCommunicationProtocols
						.getFaxArray().length];
				for (int i = 0; i < temp.length; i++) {
					temp[i] = addCommunicationProtocols.getFaxArray(i)
							.toString();
				}

				String[] newer = new String[fax.length + temp.length];
				for (int i = 0; i <= fax.length - 1; i++) {
					newer[i] = fax[i];
				}
				for (int j = 0; j <= temp.length - 1; j++) {
					newer[fax.length + j] = temp[j];
				}
				fax = newer;
			} else {
				String[] temp = new String[addCommunicationProtocols
						.getFaxArray().length];
				for (int i = 0; i < temp.length; i++) {
					temp[i] = addCommunicationProtocols.getFaxArray(i)
							.toString();
				}
				fax = temp;
			}
		}
		if (addCommunicationProtocols.sizeOfPhoneArray() > 0) {
			if (phone != null) {
				String[] temp = new String[addCommunicationProtocols
						.getPhoneArray().length];
				for (int i = 0; i < temp.length; i++) {
					temp[i] = addCommunicationProtocols.getPhoneArray(i)
							.toString();
				}
				String[] newer = new String[phone.length + temp.length];
				for (int i = 0; i <= phone.length - 1; i++) {
					newer[i] = phone[i];
				}
				for (int j = 0; j <= temp.length - 1; j++) {
					newer[phone.length + j] = temp[j];
				}
				phone = newer;
			} else {
				String[] temp = new String[addCommunicationProtocols
						.getPhoneArray().length];
				for (int i = 0; i < temp.length; i++) {
					temp[i] = addCommunicationProtocols.getPhoneArray(i)
							.toString();
				}
				phone = temp;
			}
		}

		if (addCommunicationProtocols.sizeOfSMSArray() > 0) {
			if (sms != null) {
				String[] temp = new String[addCommunicationProtocols
						.getSMSArray().length];
				for (int i = 0; i < temp.length; i++) {
					temp[i] = addCommunicationProtocols.getSMSArray(i)
							.toString();
				}
				String[] newer = new String[sms.length + temp.length];
				for (int i = 0; i <= sms.length - 1; i++) {
					newer[i] = sms[i];
				}
				for (int j = 0; j <= temp.length - 1; j++) {
					newer[sms.length + j] = temp[j];
				}
				sms = newer;
			} else {
				String[] temp = new String[addCommunicationProtocols
						.getSMSArray().length];
				for (int i = 0; i < temp.length; i++) {
					temp[i] = addCommunicationProtocols.getSMSArray(i)
							.toString();
				}
				sms = temp;
			}
		}
		if (addCommunicationProtocols.sizeOfXMPPArray() > 0) {
			if (xmpp != null) {
				String[] add = addCommunicationProtocols.getXMPPArray();
				String[] newer = new String[xmpp.length + add.length];
				for (int i = 0; i <= xmpp.length - 1; i++) {
					newer[i] = xmpp[i];
				}
				for (int j = 0; j <= add.length - 1; j++) {
					newer[xmpp.length + j] = add[j];
				}
				xmpp = newer;
			} else {
				xmpp = addCommunicationProtocols.getXMPPArray();
			}
		}
		if (addCommunicationProtocols.sizeOfHTTPArray() > 0) {
			if (http != null) {
				String[] add = addCommunicationProtocols.getHTTPArray();
				String[] newer = new String[http.length + add.length];
				for (int i = 0; i <= http.length - 1; i++) {
					newer[i] = http[i];
				}
				for (int j = 0; j <= add.length - 1; j++) {
					newer[http.length + j] = add[j];
				}
				http = newer;
			} else {
				http = addCommunicationProtocols.getHTTPArray();
			}
		}
	}

	/**
	 * 
	 * @param removeUser
	 *            The user that should be removed from this MultiUser
	 * @return A String array of the users that are not valid, null if every
	 *         user is valid
	 */
	public String[] removeMultiUser(RemoveUser removeUser) {
		int size = removeUser.sizeOfIDArray();
		int found = 0, ok = 0;
		boolean foundbool = false;
		Vector<String> notFound = new Vector<String>();
		for (int i = 0; i < removeUser.sizeOfIDArray(); i++) {
			foundbool = false;
			for (int j = 0; j < multiUserID.length; j++) {
				if (removeUser.getIDArray(i).equalsIgnoreCase(multiUserID[j])) {
					multiUserID[j] = "";
					foundbool = true;
					ok++;
				}
			}
			if (!foundbool) {
				notFound.add(found, removeUser.getIDArray(i));
				found++;
			}
		}
		if (ok != size) {
			notFound.trimToSize();
			String[] temp = new String[notFound.size()];
			for (int i = 0; i < notFound.size(); i++) {
				temp[i] = notFound.get(i);
			}
			// Remove every empty entry
			String[] tempString = new String[multiUserID.length];
			int tempInt = 0;
			for (int i = 0; i < multiUserID.length; i++) {
				if (!multiUserID[i].equalsIgnoreCase("")) {
					tempString[tempInt] = multiUserID[i];
					tempInt++;
				}
			}
			multiUserID = new String[tempInt];
			for (int i = 0; i < tempInt; i++) {
				multiUserID[i] = tempString[i];
			}

			return multiUserID;
		}
		// Remove every empty entry
		String[] tempString = new String[multiUserID.length];
		int tempInt = 0;
		for (int i = 0; i < multiUserID.length; i++) {
			if (!multiUserID[i].equalsIgnoreCase("")) {
				tempString[tempInt] = multiUserID[i];
				tempInt++;
			}
		}
		multiUserID = new String[tempInt];
		for (int i = 0; i < tempInt; i++) {
			multiUserID[i] = tempString[i];
		}

		return null;
	}

	/**
	 * 
	 * @param addUser
	 *            The user that should be added
	 */
	public void addMultiUser(AddUser addUser) {
		// Workaround with arrays, not fine!!!
		Vector<Integer> vector = new Vector<Integer>();
		for (int i = 0; i < multiUserID.length; i++) {
			for (int j = 0; j < addUser.sizeOfIDArray(); j++) {
				try {
					if (multiUserID[i].equalsIgnoreCase(addUser.getIDArray(j))) {
						vector.addElement(j);
					}
				} catch (Exception e) {
				}

			}
		}

		Vector<String> newer = new Vector<String>();
		for (int i = 0; i < multiUserID.length; i++) {
			newer.addElement(multiUserID[i]);
		}
		for (int i = 0; i < addUser.sizeOfIDArray(); i++) {
			if (!vector.contains(i)) {
				newer.addElement(addUser.getIDArray(i));
			}
		}
		// Workaround
		multiUserID = new String[newer.size()];
		for (int i = 0; i < newer.size(); i++) {
			multiUserID[i] = newer.get(i);
		}
	}

	/**
	 * 
	 * @param id
	 *            The user that should be removed
	 */
	public void removeMultiUser(String id) {
		for (int j = 0; j < multiUserID.length; j++) {
			if (multiUserID[j].equalsIgnoreCase(id)) {
				multiUserID[j] = "";
			}
		}
		// Remove every empty entry
		String[] tempString = new String[multiUserID.length];
		int tempInt = 0;
		for (int i = 0; i < multiUserID.length; i++) {
			if (!multiUserID[i].equalsIgnoreCase("")) {
				tempString[tempInt] = multiUserID[i];
				tempInt++;
			}
		}
		multiUserID = new String[tempInt];
		for (int i = 0; i < tempInt; i++) {
			multiUserID[i] = tempString[i];
		}
	}

}
