/******************************************************************************
 Web Notification Service (WNS): An open interface for a service
 by which a client may conduct asynchronous dialogues
 (message interchanges) with one or more other services.
 *******************************************************************************/
package org.n52.wns;

/**
 * This class provides all the methods to handle the user data
 * 
 * @author Dennis Dahlmann
 * @version 1.0
 */

import java.util.Hashtable;
import java.util.Iterator;
import java.util.Vector;

import net.opengis.wns.RegisterDocument;
import net.opengis.wns.RegisterMultiUserType;
import net.opengis.wns.RegisterUserResponseDocument;
import net.opengis.wns.UnregisterDocument;
import net.opengis.wns.UnregisterResponseDocument;
import net.opengis.wns.UnregisterResponseType;
import net.opengis.wns.UpdateMultiUserRegistrationDocument;
import net.opengis.wns.UpdateMultiUserRegistrationResponseDocument;
import net.opengis.wns.UpdateMultiUserRegistrationResponseType;
import net.opengis.wns.UpdateSingleUserRegistrationDocument;
import net.opengis.wns.UpdateSingleUserRegistrationResponseDocument;
import net.opengis.wns.UpdateSingleUserRegistrationResponseType.Status;

import org.n52.wns.WNSServiceException.ExceptionCode;
import org.n52.wns.WNSServiceException.ExceptionLevel;
import org.n52.wns.db.DAOFactory;
import org.n52.wns.db.UserDAO;
import org.x52North.wns.WNSUserDocument;
import org.x52North.wns.WNSUserDocument.WNSUser.SingleUser.User.Channel;

public class WNSUserHandler {

	private static WNSUserHandler instance = null;

	private UserDAO userDAO;

	private Hashtable<Long, WNSUser> singleHashthable = new Hashtable<Long, WNSUser>(); // Table

	// to
	// store

	// the UserID to the

	// user ID:WNSUser

	private Hashtable<Long, WNSUser> multiUserHashtable = new Hashtable<Long, WNSUser>(); // Table

	// to
	// store

	// the UserID to the

	// associated user ID:associated IDs

	private long userIDs = 0;

	private Vector<Long> freeUserIDs = new Vector<Long>();

	private WNSUserDocument userdoc;

	/**
	 * Constructor
	 * 
	 */
	private WNSUserHandler() {
		// TODO
		WNSInitParamContainer initParams = WNSInitParamContainer.getInstance();
		DAOFactory daofac = initParams.getDAOFactory();
		userDAO = daofac.getUserDAO();
		try {
			this.userdoc = userDAO.getUserDocument();
		} catch (WNSException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
		initUsers();
	}

	/**
	 * Singleton
	 * 
	 * @return
	 */
	public static synchronized WNSUserHandler getInstance() {
		if (instance == null) {
			instance = new WNSUserHandler();
		}
		return instance;
	}

	/**
	 * MAIN function to register a user
	 * 
	 * @param rd
	 *            The XML document, see the schema
	 * @return
	 * @throws WNSServiceException
	 */
	public synchronized RegisterUserResponseDocument register(
			RegisterDocument rd) throws WNSServiceException {
		long id = ++userIDs;

		// SingleUser
		if (rd.getRegister().isSetSingleUser()) {
			RegisterUserResponseDocument rurd = RegisterUserResponseDocument.Factory
					.newInstance();
			WNSUser wnsUser = new WNSUser(rd.getRegister().getSingleUser(), id);
			WNSUserDocument.WNSUser singleuserdoc = userdoc.getWNSUser();
			WNSUserDocument.WNSUser.SingleUser single = singleuserdoc
					.getSingleUser();
			WNSUserDocument.WNSUser.SingleUser.User singleUser = single
					.addNewUser();
			singleUser.setID(String.valueOf(wnsUser.getUserID()));
			singleUser.setName(wnsUser.getName());
			singleUser.setChannelArray(wnsUser.getChannels());

			try {
				userDAO.storeWNSUserDocument(userdoc);
			} catch (WNSException e) {
				WNSServiceException service = new WNSServiceException(
						ExceptionLevel.PlainExceptions);
				service.addCodedException(
						ExceptionCode.ErrorDuringStoringUserFile,
						"WNSUserhandler register");
				throw service;
			}
			singleHashthable.put(new Long(wnsUser.getUserID()), wnsUser);
			rurd.addNewRegisterUserResponse().setUserID(Long.toString(id));
			return rurd;

			// MultiUser
		} else {
			RegisterUserResponseDocument rurd = RegisterUserResponseDocument.Factory
					.newInstance();
			RegisterMultiUserType multi = rd.getRegister().getMultiUser();
			for (int i = 0; i < rd.getRegister().getMultiUser()
					.sizeOfUserIDArray(); i++) {
				if (singleHashthable.get(Long.parseLong(rd.getRegister()
						.getMultiUser().getUserIDArray(i))) == null) {
					multi.removeUserID(i);
				}
			}

			WNSUser wnsUser = new WNSUser(multi, id);
			WNSUserDocument.WNSUser user = userdoc.getWNSUser();
			WNSUserDocument.WNSUser.MultiUser multiUser = user.getMultiUser();
			WNSUserDocument.WNSUser.MultiUser.User multuser = multiUser
					.addNewUser();
			multuser.setID(String.valueOf(wnsUser.getUserID()));
			multuser.setAssociatedUsers(wnsUser.getAssociatedUsers());
			try {
				userDAO.storeWNSUserDocument(userdoc);
			} catch (WNSException e) {
				WNSServiceException service = new WNSServiceException(
						ExceptionLevel.PlainExceptions);
				service.addCodedException(
						ExceptionCode.ErrorDuringStoringUserFile,
						"WNSUserhandler register");
				throw service;
			}
			multiUserHashtable.put(new Long(wnsUser.getUserID()), wnsUser);
			rurd.addNewRegisterUserResponse().setUserID(Long.toString(id));
			return rurd;

		}
	}

	/**
	 * MAIN function to update the SingleUser, e.g. edit name
	 * 
	 * @param usurd
	 *            The XML dochument
	 * @return
	 * @throws WNSServiceException
	 */
	public synchronized UpdateSingleUserRegistrationResponseDocument updateSingleUser(
			UpdateSingleUserRegistrationDocument usurd)
			throws WNSServiceException {
		UpdateSingleUserRegistrationResponseDocument usurrd = UpdateSingleUserRegistrationResponseDocument.Factory
				.newInstance();
		// Update Name
		if (usurd.getUpdateSingleUserRegistration().getAction()
				.isSetUpdateName()) {
			if (singleHashthable.containsKey(new Long(usurd
					.getUpdateSingleUserRegistration().getUserID()))) {
				WNSUser wnsUser = (WNSUser) singleHashthable.get(new Long(usurd
						.getUpdateSingleUserRegistration().getUserID()));
				wnsUser.setName(usurd.getUpdateSingleUserRegistration()
						.getAction().getUpdateName());
				WNSUserDocument.WNSUser user = userdoc.getWNSUser();
				org.x52North.wns.WNSUserDocument.WNSUser.SingleUser single = user
						.getSingleUser();
				for (int i = 0; i <= single.sizeOfUserArray() - 1; i++) {
					if (single.getUserArray(i).getID()
							.equalsIgnoreCase(
									usurd.getUpdateSingleUserRegistration()
											.getUserID())) {
						single.getUserArray(i).setName(
								usurd.getUpdateSingleUserRegistration()
										.getAction().getUpdateName());
					}
				}
				try {
					userDAO.storeWNSUserDocument(userdoc);
				} catch (WNSException e) {
					WNSServiceException service = new WNSServiceException(
							ExceptionLevel.PlainExceptions);
					service.addCodedException(
							ExceptionCode.ErrorDuringStoringUserFile,
							"WNSUserhandler updateSingleUser");
					throw service;
				}
			} else {
				WNSServiceException s = new WNSServiceException(
						WNSServiceException.ExceptionLevel.PlainExceptions);
				s.addCodedException(
						WNSServiceException.ExceptionCode.UnknownUserID, null);
				throw s;
			}
			usurrd.addNewUpdateSingleUserRegistrationResponse().setStatus(
					Status.SUCCESSFULL);
			return usurrd;
		}

		// Remove protocol
		if (usurd.getUpdateSingleUserRegistration().getAction()
				.isSetRemoveCommunicationProtocols()) {
			usurrd.addNewUpdateSingleUserRegistrationResponse().setStatus(
					Status.SUCCESSFULL);
			Vector<Integer> removingChannels = new Vector<Integer>();
			if (singleHashthable.containsKey(new Long(usurd
					.getUpdateSingleUserRegistration().getUserID()))) {
				WNSUser wnsUser = (WNSUser) singleHashthable.get(new Long(usurd
						.getUpdateSingleUserRegistration().getUserID()));
				wnsUser.removeProtocol(usurd.getUpdateSingleUserRegistration()
						.getAction().getRemoveCommunicationProtocols());
				WNSUserDocument.WNSUser user = userdoc.getWNSUser();
				org.x52North.wns.WNSUserDocument.WNSUser.SingleUser single = user
						.getSingleUser();
				for (int i = 0; i < single.sizeOfUserArray(); i++) {
					// Find the user
					if (single.getUserArray(i).getID()
							.equalsIgnoreCase(
									usurd.getUpdateSingleUserRegistration()
											.getUserID())) {
						for (int j = 0; j < single.getUserArray(i)
								.sizeOfChannelArray(); j++) {
							// EMAIL
							if (single.getUserArray(i).getChannelArray(j)
									.getProtocol().equals(
											Channel.Protocol.EMAIL)
									&& usurd.getUpdateSingleUserRegistration()
											.getAction()
											.getRemoveCommunicationProtocols()
											.sizeOfEmailArray() > 0) {
								// Only one target?, delete the complete element
								if (single.getUserArray(i).getChannelArray(j)
										.sizeOfTargetArray() == 1) {
									removingChannels.addElement(j);
								} else {
									for (int k = 0; k < single.getUserArray(i)
											.getChannelArray(j)
											.sizeOfTargetArray(); k++) {
										for (int index = 0; index < usurd
												.getUpdateSingleUserRegistration()
												.getAction()
												.getRemoveCommunicationProtocols()
												.sizeOfEmailArray(); index++) {
											if (single
													.getUserArray(i)
													.getChannelArray(j)
													.getTargetArray(k)
													.equalsIgnoreCase(
															usurd
																	.getUpdateSingleUserRegistration()
																	.getAction()
																	.getRemoveCommunicationProtocols()
																	.getEmailArray(
																			index))) {
												single.getUserArray(i)
														.getChannelArray(j)
														.removeTarget(k);
											}
										}
									}
								}
							}
							// FAX
							if (single.getUserArray(i).getChannelArray(j)
									.getProtocol().equals(Channel.Protocol.FAX)
									&& usurd.getUpdateSingleUserRegistration()
											.getAction()
											.getRemoveCommunicationProtocols()
											.sizeOfFaxArray() > 0) {
								// Only one element?
								if (single.getUserArray(i).getChannelArray(j)
										.sizeOfTargetArray() == 1) {
									removingChannels.addElement(j);

								} else {

									String[] remove = new String[usurd
											.getUpdateSingleUserRegistration()
											.getAction()
											.getRemoveCommunicationProtocols()
											.sizeOfFaxArray()];
									for (int x = 0; x < remove.length; x++) {
										remove[x] = usurd
												.getUpdateSingleUserRegistration()
												.getAction()
												.getRemoveCommunicationProtocols()
												.getFaxArray(x).toString();
									}
									for (int k = 0; k < single.getUserArray(i)
											.getChannelArray(j)
											.sizeOfTargetArray(); k++) {
										for (int index = 0; index < usurd
												.getUpdateSingleUserRegistration()
												.getAction()
												.getRemoveCommunicationProtocols()
												.sizeOfFaxArray(); index++) {
											if (single.getUserArray(i)
													.getChannelArray(j)
													.getTargetArray(k)
													.equalsIgnoreCase(
															remove[index])) {
												single.getUserArray(i)
														.getChannelArray(j)
														.removeTarget(k);
											}
										}
									}
								}
							}

							// PHONE
							if (single.getUserArray(i).getChannelArray(j)
									.getProtocol().equals(
											Channel.Protocol.PHONE)
									&& usurd.getUpdateSingleUserRegistration()
											.getAction()
											.getRemoveCommunicationProtocols()
											.sizeOfPhoneArray() > 0) {
								if (single.getUserArray(i).getChannelArray(j)
										.sizeOfTargetArray() == 1) {
									removingChannels.addElement(j);
								} else {
									String[] remove = new String[usurd
											.getUpdateSingleUserRegistration()
											.getAction()
											.getRemoveCommunicationProtocols()
											.sizeOfPhoneArray()];
									for (int x = 0; x < remove.length; x++) {
										remove[x] = usurd
												.getUpdateSingleUserRegistration()
												.getAction()
												.getRemoveCommunicationProtocols()
												.getPhoneArray(x).toString();
									}
									for (int k = 0; k < single.getUserArray(i)
											.getChannelArray(j)
											.sizeOfTargetArray(); k++) {
										for (int index = 0; index < usurd
												.getUpdateSingleUserRegistration()
												.getAction()
												.getRemoveCommunicationProtocols()
												.sizeOfPhoneArray(); index++) {
											if (single.getUserArray(i)
													.getChannelArray(j)
													.getTargetArray(k)
													.equalsIgnoreCase(
															remove[index])) {
												single.getUserArray(i)
														.getChannelArray(j)
														.removeTarget(k);
											}
										}
									}
								}
							}
							// SMS
							if (single.getUserArray(i).getChannelArray(j)
									.getProtocol().equals(Channel.Protocol.SMS)
									&& usurd.getUpdateSingleUserRegistration()
											.getAction()
											.getRemoveCommunicationProtocols()
											.sizeOfSMSArray() > 0) {

								if (single.getUserArray(i).getChannelArray(j)
										.sizeOfTargetArray() == 1) {
									removingChannels.addElement(j);
								} else {
									String[] remove = new String[usurd
											.getUpdateSingleUserRegistration()
											.getAction()
											.getRemoveCommunicationProtocols()
											.sizeOfSMSArray()];
									for (int x = 0; x < remove.length; x++) {
										remove[x] = usurd
												.getUpdateSingleUserRegistration()
												.getAction()
												.getRemoveCommunicationProtocols()
												.getSMSArray(x).toString();
									}
									for (int k = 0; k < single.getUserArray(i)
											.getChannelArray(j)
											.sizeOfTargetArray(); k++) {
										for (int index = 0; index < usurd
												.getUpdateSingleUserRegistration()
												.getAction()
												.getRemoveCommunicationProtocols()
												.sizeOfSMSArray(); index++) {
											if (single.getUserArray(i)
													.getChannelArray(j)
													.getTargetArray(k)
													.equalsIgnoreCase(
															remove[index])) {
												single.getUserArray(i)
														.getChannelArray(j)
														.removeTarget(k);
											}
										}
									}
								}
							}
							// XMPP
							if (single.getUserArray(i).getChannelArray(j)
									.getProtocol()
									.equals(Channel.Protocol.XMPP)
									&& usurd.getUpdateSingleUserRegistration()
											.getAction()
											.getRemoveCommunicationProtocols()
											.sizeOfXMPPArray() > 0) {
								if (single.getUserArray(i).getChannelArray(j)
										.sizeOfTargetArray() == 1) {
									removingChannels.addElement(j);
								} else {

									for (int k = 0; k < single.getUserArray(i)
											.getChannelArray(j)
											.sizeOfTargetArray(); k++) {
										for (int index = 0; index < usurd
												.getUpdateSingleUserRegistration()
												.getAction()
												.getRemoveCommunicationProtocols()
												.sizeOfXMPPArray(); index++) {
											if (single
													.getUserArray(i)
													.getChannelArray(j)
													.getTargetArray(k)
													.equalsIgnoreCase(
															usurd
																	.getUpdateSingleUserRegistration()
																	.getAction()
																	.getRemoveCommunicationProtocols()
																	.getXMPPArray(
																			index))) {
												single.getUserArray(i)
														.getChannelArray(j)
														.removeTarget(k);
											}
										}
									}
								}
							}
							// HTTP
							if (single.getUserArray(i).getChannelArray(j)
									.getProtocol()
									.equals(Channel.Protocol.HTTP)
									&& usurd.getUpdateSingleUserRegistration()
											.getAction()
											.getRemoveCommunicationProtocols()
											.sizeOfHTTPArray() > 0) {
								if (single.getUserArray(i).getChannelArray(j)
										.sizeOfTargetArray() == 1) {
									removingChannels.addElement(j);
								} else {

									for (int k = 0; k < single.getUserArray(i)
											.getChannelArray(j)
											.sizeOfTargetArray(); k++) {
										for (int index = 0; index < usurd
												.getUpdateSingleUserRegistration()
												.getAction()
												.getRemoveCommunicationProtocols()
												.sizeOfHTTPArray(); index++) {
											if (single
													.getUserArray(i)
													.getChannelArray(j)
													.getTargetArray(k)
													.equalsIgnoreCase(
															usurd
																	.getUpdateSingleUserRegistration()
																	.getAction()
																	.getRemoveCommunicationProtocols()
																	.getHTTPArray(
																			index))) {
												single.getUserArray(i)
														.getChannelArray(j)
														.removeTarget(k);
											}
										}
									}
								}
							}
						}
						removingChannels.trimToSize();
						// Workaround to remove the channels from the last to
						// the first element, otherwise a NULLPOINTER Exception
						// occurs

						// for (int z = 0; z < removingChannels.size(); z++) {
						for (int z = removingChannels.size() - 1; z >= 0; z--) {
							single.getUserArray(i).removeChannel(
									removingChannels.get(z));
						}
					}

				}
				try {
					userDAO.storeWNSUserDocument(userdoc);
				} catch (WNSException e) {
					WNSServiceException service = new WNSServiceException(
							ExceptionLevel.PlainExceptions);
					service.addCodedException(
							ExceptionCode.ErrorDuringStoringUserFile,
							"WNSUserhandler updateSingleUser");
					throw service;
				}

				return usurrd;
			} else {
				WNSServiceException s = new WNSServiceException(
						WNSServiceException.ExceptionLevel.PlainExceptions);
				s.addCodedException(
						WNSServiceException.ExceptionCode.InvalidUserID, null);
				throw s;
			}
		}
		// Add protocol
		if (usurd.getUpdateSingleUserRegistration().getAction()
				.isSetAddCommunicationProtocols()) {
			if (singleHashthable.containsKey(new Long(usurd
					.getUpdateSingleUserRegistration().getUserID()))) {

				WNSUser wnsUser = (WNSUser) singleHashthable.get(new Long(usurd
						.getUpdateSingleUserRegistration().getUserID()));
				wnsUser.addProtocol(usurd.getUpdateSingleUserRegistration()
						.getAction().getAddCommunicationProtocols());

				WNSUserDocument.WNSUser user = userdoc.getWNSUser();
				org.x52North.wns.WNSUserDocument.WNSUser.SingleUser single = user
						.getSingleUser();
				for (int i = 0; i < single.sizeOfUserArray(); i++) {
					if (single.getUserArray(i).getID()
							.equalsIgnoreCase(
									usurd.getUpdateSingleUserRegistration()
											.getUserID())) {

						// IF the user has already channels
						if (single.getUserArray(i).getChannelArray().length > 0) {
							// EMAIL
							if (usurd.getUpdateSingleUserRegistration()
									.getAction().getAddCommunicationProtocols()
									.sizeOfEmailArray() > 0) {
								boolean found = false;
								for (int j = 0; j < single.getUserArray(i)
										.getChannelArray().length; j++) {
									if (single.getUserArray(i).getChannelArray(
											j).getProtocol().equals(
											Channel.Protocol.EMAIL)) {
										found = true;
										for (int k = 0; k < usurd
												.getUpdateSingleUserRegistration()
												.getAction()
												.getAddCommunicationProtocols()
												.sizeOfEmailArray(); k++) {
											single
													.getUserArray(i)
													.getChannelArray(j)
													.addTarget(
															usurd
																	.getUpdateSingleUserRegistration()
																	.getAction()
																	.getAddCommunicationProtocols()
																	.getEmailArray(
																			k));
										}

									}
								}
								if (!found) {
									Channel chan = single.getUserArray(i)
											.addNewChannel();
									chan.setProtocol(Channel.Protocol.EMAIL);
									chan.setTargetArray(usurd
											.getUpdateSingleUserRegistration()
											.getAction()
											.getAddCommunicationProtocols()
											.getEmailArray());

								}

							}
							// FAX
							if (usurd.getUpdateSingleUserRegistration()
									.getAction().getAddCommunicationProtocols()
									.sizeOfFaxArray() > 0) {
								boolean found = false;

								for (int j = 0; j < single.getUserArray(i)
										.getChannelArray().length; j++) {
									if (single.getUserArray(i).getChannelArray(
											j).getProtocol().equals(
											Channel.Protocol.FAX)) {
										found = true;
										for (int x = 0; x < usurd
												.getUpdateSingleUserRegistration()
												.getAction()
												.getAddCommunicationProtocols()
												.sizeOfFaxArray(); x++) {
											single
													.getUserArray(i)
													.getChannelArray(j)
													.addTarget(
															usurd
																	.getUpdateSingleUserRegistration()
																	.getAction()
																	.getAddCommunicationProtocols()
																	.getFaxArray(
																			x)
																	.toString());
										}
									}
								}

								if (!found) {
									Channel chan = single.getUserArray(i)
											.addNewChannel();
									chan.setProtocol(Channel.Protocol.FAX);
									for (int j = 0; j < usurd
											.getUpdateSingleUserRegistration()
											.getAction()
											.getAddCommunicationProtocols()
											.sizeOfFaxArray(); j++) {
										chan
												.addTarget(usurd
														.getUpdateSingleUserRegistration()
														.getAction()
														.getAddCommunicationProtocols()
														.getFaxArray(j)
														.toString());
									}

								}

							}
							// SMS
							if (usurd.getUpdateSingleUserRegistration()
									.getAction().getAddCommunicationProtocols()
									.sizeOfSMSArray() > 0) {
								boolean found = false;
								for (int j = 0; j < single.getUserArray(i)
										.getChannelArray().length; j++) {
									if (single.getUserArray(i).getChannelArray(
											j).getProtocol().equals(
											Channel.Protocol.SMS)) {
										found = true;
										for (int x = 0; x < usurd
												.getUpdateSingleUserRegistration()
												.getAction()
												.getAddCommunicationProtocols()
												.getSMSArray().length; x++) {
											single
													.getUserArray(i)
													.getChannelArray(j)
													.addTarget(
															usurd
																	.getUpdateSingleUserRegistration()
																	.getAction()
																	.getAddCommunicationProtocols()
																	.getSMSArray(
																			x)
																	.toString());
										}
									}
								}
								if (!found) {
									Channel chan = single.getUserArray(i)
											.addNewChannel();
									chan.setProtocol(Channel.Protocol.SMS);

									for (int j = 0; j < usurd
											.getUpdateSingleUserRegistration()
											.getAction()
											.getAddCommunicationProtocols()
											.sizeOfSMSArray(); j++) {
										chan
												.addTarget(usurd
														.getUpdateSingleUserRegistration()
														.getAction()
														.getAddCommunicationProtocols()
														.getSMSArray(j)
														.toString());
									}

								}

							}
							// PHONE
							if (usurd.getUpdateSingleUserRegistration()
									.getAction().getAddCommunicationProtocols()
									.sizeOfPhoneArray() > 0) {
								boolean found = false;
								for (int j = 0; j < single.getUserArray(i)
										.getChannelArray().length; j++) {
									if (single.getUserArray(i).getChannelArray(
											j).getProtocol().equals(
											Channel.Protocol.PHONE)) {
										found = true;
										for (int x = 0; x < usurd
												.getUpdateSingleUserRegistration()
												.getAction()
												.getAddCommunicationProtocols()
												.sizeOfPhoneArray(); x++) {
											single
													.getUserArray(i)
													.getChannelArray(j)
													.addTarget(
															usurd
																	.getUpdateSingleUserRegistration()
																	.getAction()
																	.getAddCommunicationProtocols()
																	.getPhoneArray(
																			x)
																	.toString());
										}
									}
								}
								if (!found) {
									Channel chan = single.getUserArray(i)
											.addNewChannel();
									chan.setProtocol(Channel.Protocol.PHONE);
									for (int j = 0; j < usurd
											.getUpdateSingleUserRegistration()
											.getAction()
											.getAddCommunicationProtocols()
											.sizeOfPhoneArray(); j++) {
										chan
												.addTarget(usurd
														.getUpdateSingleUserRegistration()
														.getAction()
														.getAddCommunicationProtocols()
														.getPhoneArray(j)
														.toString());
									}

								}

							}
							// XMMP
							if (usurd.getUpdateSingleUserRegistration()
									.getAction().getAddCommunicationProtocols()
									.sizeOfXMPPArray() > 0) {
								boolean found = false;
								for (int j = 0; j < single.getUserArray(i)
										.getChannelArray().length; j++) {
									if (single.getUserArray(i).getChannelArray(
											j).getProtocol().equals(
											Channel.Protocol.XMPP)) {
										found = true;
										for (int k = 0; k < usurd
												.getUpdateSingleUserRegistration()
												.getAction()
												.getAddCommunicationProtocols()
												.sizeOfXMPPArray(); k++) {
											single
													.getUserArray(i)
													.getChannelArray(j)
													.addTarget(
															usurd
																	.getUpdateSingleUserRegistration()
																	.getAction()
																	.getAddCommunicationProtocols()
																	.getXMPPArray(
																			k));
										}

									}
								}
								if (!found) {
									Channel chan = single.getUserArray(i)
											.addNewChannel();
									chan.setProtocol(Channel.Protocol.XMPP);
									chan.setTargetArray(usurd
											.getUpdateSingleUserRegistration()
											.getAction()
											.getAddCommunicationProtocols()
											.getXMPPArray());

								}
							}
							// HTTP
							if (usurd.getUpdateSingleUserRegistration()
									.getAction().getAddCommunicationProtocols()
									.sizeOfHTTPArray() > 0) {
								boolean found = false;
								for (int j = 0; j < single.getUserArray(i)
										.getChannelArray().length; j++) {
									if (single.getUserArray(i).getChannelArray(
											j).getProtocol().equals(
											Channel.Protocol.HTTP)) {
										found = true;
										for (int k = 0; k < usurd
												.getUpdateSingleUserRegistration()
												.getAction()
												.getAddCommunicationProtocols()
												.sizeOfHTTPArray(); k++) {
											single
													.getUserArray(i)
													.getChannelArray(j)
													.addTarget(
															usurd
																	.getUpdateSingleUserRegistration()
																	.getAction()
																	.getAddCommunicationProtocols()
																	.getHTTPArray(
																			k));
										}

									}
								}
								if (!found) {
									Channel chan = single.getUserArray(i)
											.addNewChannel();
									chan.setProtocol(Channel.Protocol.HTTP);
									chan.setTargetArray(usurd
											.getUpdateSingleUserRegistration()
											.getAction()
											.getAddCommunicationProtocols()
											.getHTTPArray());

								}
							}
						} else {
							if (usurd.getUpdateSingleUserRegistration()
									.getAction().getAddCommunicationProtocols()
									.sizeOfEmailArray() > 0) {
								Channel chan = single.getUserArray(i)
										.addNewChannel();
								chan.setProtocol(Channel.Protocol.EMAIL);
								chan.setTargetArray(usurd
										.getUpdateSingleUserRegistration()
										.getAction()
										.getAddCommunicationProtocols()
										.getEmailArray());
							}
							if (usurd.getUpdateSingleUserRegistration()
									.getAction().getAddCommunicationProtocols()
									.sizeOfFaxArray() > 0) {
								Channel chan = single.getUserArray(i)
										.addNewChannel();
								chan.setProtocol(Channel.Protocol.FAX);
								String[] temp = new String[usurd
										.getUpdateSingleUserRegistration()
										.getAction()
										.getAddCommunicationProtocols()
										.sizeOfFaxArray()];
								for (int j = 0; j < temp.length; j++) {
									temp[j] = usurd
											.getUpdateSingleUserRegistration()
											.getAction()
											.getAddCommunicationProtocols()
											.getFaxArray(j).toString();
								}
								chan.setTargetArray(temp);
							}
							if (usurd.getUpdateSingleUserRegistration()
									.getAction().getAddCommunicationProtocols()
									.sizeOfSMSArray() > 0) {
								Channel chan = single.getUserArray(i)
										.addNewChannel();
								chan.setProtocol(Channel.Protocol.SMS);
								String[] temp = new String[usurd
										.getUpdateSingleUserRegistration()
										.getAction()
										.getAddCommunicationProtocols()
										.sizeOfSMSArray()];
								for (int j = 0; j < temp.length; j++) {
									temp[j] = usurd
											.getUpdateSingleUserRegistration()
											.getAction()
											.getAddCommunicationProtocols()
											.getSMSArray(j).toString();
								}
								chan.setTargetArray(temp);
							}
							if (usurd.getUpdateSingleUserRegistration()
									.getAction().getAddCommunicationProtocols()
									.sizeOfPhoneArray() > 0) {
								Channel chan = single.getUserArray(i)
										.addNewChannel();
								chan.setProtocol(Channel.Protocol.PHONE);
								String[] temp = new String[usurd
										.getUpdateSingleUserRegistration()
										.getAction()
										.getAddCommunicationProtocols()
										.sizeOfPhoneArray()];
								for (int j = 0; j < temp.length; j++) {
									temp[j] = usurd
											.getUpdateSingleUserRegistration()
											.getAction()
											.getAddCommunicationProtocols()
											.getPhoneArray(j).toString();
								}
								chan.setTargetArray(temp);
							}
							if (usurd.getUpdateSingleUserRegistration()
									.getAction().getAddCommunicationProtocols()
									.sizeOfXMPPArray() > 0) {
								Channel chan = single.getUserArray(i)
										.addNewChannel();
								chan.setProtocol(Channel.Protocol.XMPP);
								chan.setTargetArray(usurd
										.getUpdateSingleUserRegistration()
										.getAction()
										.getAddCommunicationProtocols()
										.getXMPPArray());
							}
							if (usurd.getUpdateSingleUserRegistration()
									.getAction().getAddCommunicationProtocols()
									.sizeOfHTTPArray() > 0) {
								Channel chan = single.getUserArray(i)
										.addNewChannel();
								chan.setProtocol(Channel.Protocol.HTTP);
								chan.setTargetArray(usurd
										.getUpdateSingleUserRegistration()
										.getAction()
										.getAddCommunicationProtocols()
										.getHTTPArray());
							}
						}
						i = single.sizeOfUserArray() + 1;
					}
				}

				try {
					userDAO.storeWNSUserDocument(userdoc);
				} catch (WNSException e) {
					WNSServiceException service = new WNSServiceException(
							ExceptionLevel.PlainExceptions);
					service.addCodedException(
							ExceptionCode.ErrorDuringStoringUserFile,
							"WNSUserhandler updateSingleUser");
					throw service;
				}
				usurrd.addNewUpdateSingleUserRegistrationResponse().setStatus(
						Status.SUCCESSFULL);
				return usurrd;
			} else {
				WNSServiceException s = new WNSServiceException(
						WNSServiceException.ExceptionLevel.PlainExceptions);
				s.addCodedException(
						WNSServiceException.ExceptionCode.InvalidUserID, null);
				throw s;
			}

		}
		return null;
	}

	/**
	 * MAIN function to update a MultiUser
	 * 
	 * @param umurd
	 *            The XML document
	 * @return
	 * @throws WNSServiceException
	 */
	public synchronized UpdateMultiUserRegistrationResponseDocument updateMultiUser(
			UpdateMultiUserRegistrationDocument umurd)
			throws WNSServiceException {
		UpdateMultiUserRegistrationResponseDocument response = UpdateMultiUserRegistrationResponseDocument.Factory
				.newInstance();
		response.addNewUpdateMultiUserRegistrationResponse().setStatus(
				UpdateMultiUserRegistrationResponseType.Status.SUCCESSFULL);
		WNSUserDocument.WNSUser user = userdoc.getWNSUser();
		// Updating the hashtable
		if (multiUserHashtable.containsKey(new Long(umurd
				.getUpdateMultiUserRegistration().getMultiUserID()))) {
			WNSUser wnsUser = (WNSUser) multiUserHashtable.get(new Long(umurd
					.getUpdateMultiUserRegistration().getMultiUserID()));
			// Adding User
			if (umurd.getUpdateMultiUserRegistration().getAction()
					.isSetAddUser()) {
				String[] unknownIDs = new String[umurd
						.getUpdateMultiUserRegistration().getAction()
						.getAddUser().sizeOfIDArray()];
				int tempIndex = 0;
				for (int i = 0; i < umurd.getUpdateMultiUserRegistration()
						.getAction().getAddUser().sizeOfIDArray(); i++) {
					if (singleHashthable.containsKey(new Long(umurd
							.getUpdateMultiUserRegistration().getAction()
							.getAddUser().getIDArray(i)))) {
						i = umurd.getUpdateMultiUserRegistration().getAction()
								.getAddUser().sizeOfIDArray();
					} else {
						// if (multiUserHashtable.containsKey(new Long(umurd
						// .getUpdateMultiUserRegistration().getAction()
						// .getAddUser().getIDArray(i)))) {
						// i = umurd.getUpdateMultiUserRegistration()
						// .getAction().getAddUser().sizeOfIDArray();
						// } else {
						unknownIDs[tempIndex] = umurd
								.getUpdateMultiUserRegistration().getAction()
								.getAddUser().getIDArray(i);
						tempIndex++;

						// }
					}
				}
				if (tempIndex > 0) {
					String[] temp = new String[tempIndex];
					for (int i = 0; i < tempIndex; i++) {
						temp[i] = unknownIDs[i];
					}
					response
							.getUpdateMultiUserRegistrationResponse()
							.setStatus(
									UpdateMultiUserRegistrationResponseType.Status.SUCCESSFULL_BUT_UNKNOWN_UPDATE_ID_ENCOUNTERED);
					response.getUpdateMultiUserRegistrationResponse()
							.addNewUnknownIDs().setIDArray(temp);
					return response;
				}

				wnsUser.addMultiUser(umurd.getUpdateMultiUserRegistration()
						.getAction().getAddUser());

				// Updating the user.xml file
				org.x52North.wns.WNSUserDocument.WNSUser.MultiUser multi = user
						.getMultiUser();
				// Workaround if one of the new users are already part of the
				// multiuser

				for (int i = 0; i < multi.sizeOfUserArray(); i++) {
					if (multi.getUserArray(i).getID().equalsIgnoreCase(
							umurd.getUpdateMultiUserRegistration()
									.getMultiUserID())) {
						Vector<Integer> vector = new Vector<Integer>();
						for (int y = 0; y < multi.getUserArray(i)
								.getAssociatedUsers().sizeOfIDArray(); y++) {
							for (int x = 0; x < umurd
									.getUpdateMultiUserRegistration()
									.getAction().getAddUser().sizeOfIDArray(); x++) {
								if (multi
										.getUserArray(i)
										.getAssociatedUsers()
										.getIDArray(y)
										.equalsIgnoreCase(
												umurd
														.getUpdateMultiUserRegistration()
														.getAction()
														.getAddUser()
														.getIDArray(x))) {
									vector.addElement(x);

								}
							}
						}
						for (int j = 0; j < umurd
								.getUpdateMultiUserRegistration().getAction()
								.getAddUser().sizeOfIDArray(); j++) {
							if (!vector.contains(j)) {
								multi
										.getUserArray(i)
										.getAssociatedUsers()
										.addID(
												umurd
														.getUpdateMultiUserRegistration()
														.getAction()
														.getAddUser()
														.getIDArray(j));
							}

						}
					}
				}
			}

			// Removing User
			else {
				String[] notfound = wnsUser.removeMultiUser(umurd
						.getUpdateMultiUserRegistration().getAction()
						.getRemoveUser());
				if (notfound != null) {
					response
							.getUpdateMultiUserRegistrationResponse()
							.setStatus(
									UpdateMultiUserRegistrationResponseType.Status.SUCCESSFULL_BUT_UNKNOWN_UPDATE_ID_ENCOUNTERED);
					response.getUpdateMultiUserRegistrationResponse()
							.addNewUnknownIDs().setIDArray(notfound);
				}

				// Updating the user.xml file
				org.x52North.wns.WNSUserDocument.WNSUser.MultiUser multi = user
						.getMultiUser();
				for (int i = 0; i < multi.sizeOfUserArray(); i++) {
					if (multi.getUserArray(i).getID().equalsIgnoreCase(
							umurd.getUpdateMultiUserRegistration()
									.getMultiUserID())) {
						for (int j = 0; j < multi.getUserArray(i)
								.getAssociatedUsers().sizeOfIDArray(); j++) {
							for (int k = 0; k < umurd
									.getUpdateMultiUserRegistration()
									.getAction().getRemoveUser()
									.sizeOfIDArray(); k++) {
								if (multi
										.getUserArray(i)
										.getAssociatedUsers()
										.getIDArray(j)
										.equalsIgnoreCase(
												umurd
														.getUpdateMultiUserRegistration()
														.getAction()
														.getRemoveUser()
														.getIDArray(k))) {
									multi.getUserArray(i).getAssociatedUsers()
											.removeID(j);
								}
							}
						}
					}
				}
			}

		} else {
			// UserID not found in hashtable
			response
					.getUpdateMultiUserRegistrationResponse()
					.setStatus(
							UpdateMultiUserRegistrationResponseType.Status.UNKNOWN_MULTI_USER_ID);
			return response;
		}
		try {
			userDAO.storeWNSUserDocument(userdoc);
		} catch (WNSException e) {
			WNSServiceException service = new WNSServiceException(
					ExceptionLevel.PlainExceptions);
			service.addCodedException(ExceptionCode.ErrorDuringStoringUserFile,
					"WNSUserhandler updateSingleUser");
			throw service;
		}

		return response;
	}

	/**
	 * MAIN function to delete/ unregister a user
	 * 
	 * @param ud
	 * @return
	 * @throws WNSServiceException
	 */
	public synchronized UnregisterResponseDocument unregister(
			UnregisterDocument ud) throws WNSServiceException {
		UnregisterResponseDocument response = UnregisterResponseDocument.Factory
				.newInstance();
		WNSUserDocument.WNSUser user = userdoc.getWNSUser();
		if (singleHashthable.containsKey(new Long(ud.getUnregister().getID()))) {
			org.x52North.wns.WNSUserDocument.WNSUser.SingleUser single = user
					.getSingleUser();
			for (int i = 0; i <= single.sizeOfUserArray() - 1; i++) {
				if (single.getUserArray(i).getID().equalsIgnoreCase(
						ud.getUnregister().getID())) {
					single.removeUser(i);
					singleHashthable
							.remove(new Long(ud.getUnregister().getID()));
					checkMultiUser(ud.getUnregister().getID());
				}
			}

		} else {
			if (multiUserHashtable.containsKey(new Long(ud.getUnregister()
					.getID()))) {
				org.x52North.wns.WNSUserDocument.WNSUser.MultiUser multi = user
						.getMultiUser();
				for (int i = 0; i <= multi.sizeOfUserArray() - 1; i++) {
					if (multi.getUserArray(i).getID().equalsIgnoreCase(
							ud.getUnregister().getID())) {
						multi.removeUser(i);
						multiUserHashtable.remove(new Long(ud.getUnregister()
								.getID()));
					}
				}
			} else {
				response.addNewUnregisterResponse().setStatus(
						UnregisterResponseType.Status.UNKNOWN_ID);
				return response;
			}

		}

		try {
			userDAO.storeWNSUserDocument(userdoc);
		} catch (WNSException e) {
			WNSServiceException service = new WNSServiceException(
					ExceptionLevel.PlainExceptions);
			service.addCodedException(ExceptionCode.ErrorDuringStoringUserFile,
					"WNSUserhandler updateSingleUser");
			throw service;
		}
		// freeUserIDs.addElement(new Long(ud.getUnregister().getID()));
		response.addNewUnregisterResponse().setStatus(
				UnregisterResponseType.Status.SUCCESS);
		return response;
	}

	private void checkMultiUser(String id) {
		WNSUserDocument.WNSUser user = userdoc.getWNSUser();
		org.x52North.wns.WNSUserDocument.WNSUser.MultiUser multi = user
				.getMultiUser();
		org.x52North.wns.WNSUserDocument.WNSUser.MultiUser.User multiuser;

		// Hashtable
		Iterator elem;
		int temp = 0;
		if (multiUserHashtable.elements().hasMoreElements()) {
			elem = (Iterator) multiUserHashtable.elements();
		} else
			return;

		while (temp < multiUserHashtable.size()) {
			temp++;
			WNSUser wnsuser = (WNSUser) elem.next();
			for (int i = 0; i < wnsuser.getAssociatedUsers().sizeOfIDArray(); i++) {
				if (wnsuser.getAssociatedUsers().getIDArray(i)
						.equalsIgnoreCase(id)) {
					wnsuser.removeMultiUser(id);
				}
			}
		}
		// WNSUserDocument
		for (int i = 0; i < multi.sizeOfUserArray(); i++) {
			multiuser = multi.getUserArray(i);
			for (int j = 0; j < multiuser.getAssociatedUsers().sizeOfIDArray(); j++) {
				if (multiuser.getAssociatedUsers().getIDArray(j)
						.equalsIgnoreCase(id)) {
				}
			}

		}

	}

	/**
	 * MAIN function to get the fully XML message
	 * 
	 * @param userID
	 * @return
	 * @throws WNSException
	 */
	public synchronized Channel[] getMessageChannels(String userID)
			throws WNSException {
		if (singleHashthable.containsKey(new Long(userID))) {
			return ((WNSUser) singleHashthable.get(new Long(userID)))
					.getChannels();
		} else {
			if (multiUserHashtable.containsKey(new Long(userID))) {
				org.x52North.wns.WNSUserDocument.WNSUser.SingleUser.User user = org.x52North.wns.WNSUserDocument.WNSUser.SingleUser.User.Factory
						.newInstance();
				Channel email = user.addNewChannel();
				email.setProtocol(Channel.Protocol.EMAIL);
				Channel fax = user.addNewChannel();
				fax.setProtocol(Channel.Protocol.FAX);
				Channel phone = user.addNewChannel();
				phone.setProtocol(Channel.Protocol.PHONE);
				Channel sms = user.addNewChannel();
				sms.setProtocol(Channel.Protocol.SMS);
				Channel xmpp = user.addNewChannel();
				xmpp.setProtocol(Channel.Protocol.XMPP);
				Channel http = user.addNewChannel();
				http.setProtocol(Channel.Protocol.HTTP);
				WNSUser wnsuser = ((WNSUser) multiUserHashtable.get(new Long(
						userID)));
				for (int i = 0; i < wnsuser.getAssociatedUsers()
						.sizeOfIDArray(); i++) {
					WNSUser singleUser = (WNSUser) singleHashthable.get(Long
							.parseLong(wnsuser.getAssociatedUsers().getIDArray(
									i)));
					Channel[] chan = singleUser.getChannels();
					for (int j = 0; j < chan.length; j++) {
						if (chan[j].getProtocol()
								.equals(Channel.Protocol.EMAIL)) {
							for (int k = 0; k < chan[j].getTargetArray().length; k++) {
								email.addTarget(chan[j].getTargetArray(k));
							}
						}
						if (chan[j].getProtocol().equals(Channel.Protocol.FAX)) {
							for (int k = 0; k < chan[j].getTargetArray().length; k++) {
								fax.setTargetArray(chan[j].getTargetArray());
							}
						}
						if (chan[j].getProtocol()
								.equals(Channel.Protocol.PHONE)) {
							for (int k = 0; k < chan[j].getTargetArray().length; k++) {
								phone.setTargetArray(chan[j].getTargetArray());
							}
						}
						if (chan[j].getProtocol().equals(Channel.Protocol.SMS)) {
							for (int k = 0; k < chan[j].getTargetArray().length; k++) {
								sms.setTargetArray(chan[j].getTargetArray());
							}
						}
						if (chan[j].getProtocol().equals(Channel.Protocol.XMPP)) {
							for (int k = 0; k < chan[j].getTargetArray().length; k++) {
								xmpp.setTargetArray(chan[j].getTargetArray());
							}
						}
						if (chan[j].getProtocol().equals(Channel.Protocol.HTTP)) {
							for (int k = 0; k < chan[j].getTargetArray().length; k++) {
								http.setTargetArray(chan[j].getTargetArray());
							}
						}
					}

				}
				return user.getChannelArray();
			}
		}
		throw new WNSException("No User found") {
		};
	}

	private void initUsers() {
		for (int i = 0; i <= userdoc.getWNSUser().getSingleUser()
				.getUserArray().length - 1; i++) {
			WNSUser user = new WNSUser(userdoc.getWNSUser().getSingleUser()
					.getUserArray(i));

			if (user.getUserID() > userIDs) {
				userIDs = user.getUserID();
			}
			singleHashthable.put(new Long(user.getUserID()), user);
		}
		for (int i = 0; i <= userdoc.getWNSUser().getMultiUser().getUserArray().length - 1; i++) {
			WNSUser user = new WNSUser(userdoc.getWNSUser().getMultiUser()
					.getUserArray(i));
			if (user.getUserID() > userIDs) {
				userIDs = user.getUserID();
			}
			multiUserHashtable.put(new Long(user.getUserID()), user);
		}
	}

}
