/******************************************************************************
 Web Notification Service (WNS): An open interface for a service
 by which a client may conduct asynchronous dialogues
 (message interchanges) with one or more other services.
 *******************************************************************************/

package org.n52.wns;

import java.io.DataOutputStream;
import java.net.URL;
import java.net.URLConnection;
import java.util.logging.Logger;

import net.opengis.wns.CommunicationMessageDocument;
import net.opengis.wns.DoCommunicationDocument;
import net.opengis.wns.DoNotificationDocument;
import net.opengis.wns.NotificationMessageDocument;

/**
 * This class provides methods to send data to other servlets using HTTP POST
 * 
 * @author Dennis Dahlmann
 * @version 1.0
 */
public class WNSWebServiceHandler {

	private Logger log = Logger.getLogger(WNSWebServiceHandler.class.getName());

	private final String MIME_TYPE = "";

	/**
	 * Constructor
	 */
	public WNSWebServiceHandler() {
	}

	/**
	 * sends a notification message to another servlet using POST
	 * 
	 * @param WNSUser
	 *            user to send the message to
	 * @param WNSMessage
	 *            message to be sent
	 */
	public void sendNotificationMessage(DoNotificationDocument dnd,
			String[] targets) throws WNSServiceException {
		try {

			for (int i = 0; i <= targets.length - 1; i++) {
				// connect to the servlet
				String location = targets[i].toString();
				URL destinationServlet = new URL(location);
				URLConnection servletConnection = destinationServlet
						.openConnection();

				// inform the connection that we will send output and accept
				// input
				servletConnection.setDoInput(true);
				servletConnection.setDoOutput(true);

				// Don't use a cached version of URL connection.
				servletConnection.setUseCaches(false);
				servletConnection.setDefaultUseCaches(false);

				// Specify the content type that we will send xml data
				servletConnection.setRequestProperty("Content-Type",
						"application/xml");

				// send the message object to the servlet using serialization
				DataOutputStream outputToServlet = new DataOutputStream(
						servletConnection.getOutputStream());

				// write the message
				NotificationMessageDocument notdoc = NotificationMessageDocument.Factory
						.newInstance();
				notdoc.setNotificationMessage(dnd.getDoNotification()
						.getMessage());
				outputToServlet.writeBytes(notdoc.toString());

				outputToServlet.flush();
				outputToServlet.close();

				servletConnection.getInputStream();
			}
		} catch (Exception e) {
			e.printStackTrace();
		}
	}

	/**
	 * sends a communication message to another servlet using POST
	 * 
	 * @param WNSUser
	 *            user to send the message to
	 * @param WNSMessage
	 *            message to be sent
	 */
	public void sendCommunicationMessage(DoCommunicationDocument dcd,
			String[] targets) throws WNSServiceException {
		try {

			for (int i = 0; i <= targets.length - 1; i++) {
				// connect to the servlet
				String location = targets[i].toString();
				URL destinationServlet = new URL(location);
				URLConnection servletConnection = destinationServlet
						.openConnection();

				// inform the connection that we will send output and accept
				// input
				servletConnection.setDoInput(true);
				servletConnection.setDoOutput(true);

				// Don't use a cached version of URL connection.
				servletConnection.setUseCaches(false);
				servletConnection.setDefaultUseCaches(false);

				// Specify the content type that we will send xml data
				servletConnection.setRequestProperty("Content-Type",
						"application/xml");

				// send the message object to the servlet using serialization
				DataOutputStream outputToServlet = new DataOutputStream(
						servletConnection.getOutputStream());

				// write the message
				CommunicationMessageDocument comdoc = CommunicationMessageDocument.Factory
						.newInstance();
				comdoc.setCommunicationMessage(dcd.getDoCommunication()
						.getMessage());
				outputToServlet.writeBytes(comdoc.toString());

				outputToServlet.flush();
				outputToServlet.close();
				servletConnection.getInputStream();
			}
		} catch (Exception e) {
		}
	}
}
