/******************************************************************************
 Web Notification Service (WNS): An open interface for a service
 by which a client may conduct asynchronous dialogues
 (message interchanges) with one or more other services.

 The implementation is in compliance with
 http://www.opengis.org/docs/03-008r2.pdf

 Copyright (C) 2004 by con terra GmbH

 Author: Ingo Simonis, University of Muenster

 Contact: Albert Remke, con terra GmbH, Martin-Luther-King-Weg 24,
 48155 Muenster, Germany, 52n@conterra.de

 This program is free software; you can redistribute it and/or
 modify it under the terms of the GNU General Public License
 version 2 as published by the Free Software Foundation.

 This program is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 GNU General Public License for more details.

 You should have received a copy of the GNU General Public License
 along with this program (see gnu-gpl v2.txt); if not, write to
 the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 Boston, MA  02111-1307, USA or visit the web page of the Free
 Software Foundation, http://www.fsf.org.
 *******************************************************************************/

/*
 * Created on 27.04.2003
 *
 * Version:
 */
package org.n52.wns.communication;

import java.util.logging.Logger;

import net.opengis.wns.DoCommunicationDocument;
import net.opengis.wns.DoNotificationDocument;

import org.n52.wns.WNSException;
import org.n52.wns.WNSServiceException;
import org.n52.wns.mail.MailHandler;
import org.x52North.wns.WNSConfigDocument;

/**
 * @author Dennis Dahlmann
 * @version 1.0
 */
public class FaxHandlerEcall implements FaxHandler {

	private Logger log = Logger.getLogger(FaxHandlerEcall.class.getName());

	// Ecall's pop server
	private String host;

	private MailHandler mh;

	private String wns;

	/**
	 * constructor FaxHandlerEcall loads the MailHandler
	 * 
	 * @throws WNSException
	 */
	public FaxHandlerEcall() throws WNSException {
		mh = org.n52.wns.mail.MailHandlerFactory.getInstance();
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see org.n52.communication.FaxHandler#setProperties(org.jdom.Element)
	 */
	public void setProperties(
			WNSConfigDocument.WNSConfig.RegisteredHandlers.FaxHandler faxHandler,
			String wns) throws WNSServiceException {
		host = faxHandler.getHost();
		this.wns = wns;
	}

	/**
	 * sends a FAX NotificationMessage
	 * 
	 * @param user
	 *            The WNSUser
	 * @param message
	 *            The WNSMessage
	 */
	public void sendNotificationMessage(DoNotificationDocument dnd,
			String[] targets, String messageID) throws WNSServiceException {

		String subject;
		String body;
		String lineSep = System.getProperty("line.separator");

		for (int i = 0; i <= targets.length - 1; i++) {
			String msisdn = targets[i].toString();

			// verify format
			msisdn = PhoneNumber.convertToInternationalFormat(msisdn);

			String to = msisdn + "@" + host;

			subject = ("Web Notification Service");

			body = "Notification Message"
					+ lineSep
					+ "MessageID: "
					+ messageID
					+ lineSep
					+ "WNS URL "
					+ wns
					+ lineSep
					+ "Service URL "
					+ dnd.getDoNotification().getMessage()
							.getServiceDescription().getServiceURL() + lineSep
					+ "Message: "
					+ dnd.getDoNotification().getMessage().getShortMessage();
			System.out.println(body);
			mh.sendExternalMessage(to, subject, body);

		}
	}

	/**
	 * sends a FAX Communication Message
	 * 
	 * @param user
	 *            The WNSUser
	 * @param message
	 *            The WNSMessage
	 * @throws WNSException
	 */
	public void sendCommunicationMessage(DoCommunicationDocument dcd,
			String[] targets, String messageID) throws WNSServiceException {

		String subject;
		String body;
		String lineSep = System.getProperty("line.separator");

		for (int i = 0; i <= targets.length - 1; i++) {
			String msisdn = targets[i].toString();

			// verify format
			msisdn = PhoneNumber.convertToInternationalFormat(msisdn);

			String to = msisdn + "@" + host;

			subject = "Web Notification Service";

			body = "Communication Message "
					+ lineSep
					+ "MessageID: "
					+ messageID
					+ lineSep
					+ "WNS URL "
					+ wns
					+ lineSep
					+ "Service URL "
					+ dcd.getDoCommunication().getMessage()
							.getServiceDescription().getServiceURL() + lineSep
					+ "Message: "
					+ dcd.getDoCommunication().getMessage().getShortMessage();
			System.out.println(body);
			mh.sendExternalMessage(to, subject, body);

		}
	}
}
