/******************************************************************************
 Web Notification Service (WNS): An open interface for a service
 by which a client may conduct asynchronous dialogues
 (message interchanges) with one or more other services.
 *******************************************************************************/

package org.n52.wns.communication;

import net.opengis.wns.DoCommunicationDocument;
import net.opengis.wns.DoNotificationDocument;

import org.n52.wns.WNSException;
import org.n52.wns.WNSServiceException;
import org.x52North.wns.WNSConfigDocument;

/**
 * Interface PhoneHandler is the general interface for all phone handlers that
 * are used to forward message using telephony
 * 
 * @author Dennis Dahlmann
 * @version 1.0
 */
public interface PhoneHandler {

	/**
	 * sets the configuration properties of the phone handler
	 * 
	 * @param e
	 *            org.jdom.Element containing all config information
	 * @throws WNSException
	 */
	public void setProperties(
			WNSConfigDocument.WNSConfig.RegisteredHandlers.PhoneHandler phoneHandler,
			String wns) throws WNSServiceException;

	/**
	 * sends a message using a phone handler
	 * 
	 * @param user
	 *            recipient of the message
	 * @param message
	 *            message to be sent
	 * @throws WNSException
	 */
	public void sendCommunicationMessage(DoCommunicationDocument dcd,
			String[] targets, String messageID) throws WNSServiceException;

	/**
	 * sends a message using a phone handler
	 * 
	 * @param user
	 *            recipient of the message
	 * @param message
	 *            message to be sent
	 * @throws WNSException
	 */
	public void sendNotificationMessage(DoNotificationDocument dnd,
			String[] targets, String messageID) throws WNSServiceException;
}
