/******************************************************************************
 Web Notification Service (WNS): An open interface for a service
 by which a client may conduct asynchronous dialogues
 (message interchanges) with one or more other services.
 *******************************************************************************/

package org.n52.wns.communication;

import java.util.logging.Logger;

import net.opengis.wns.DoCommunicationDocument;
import net.opengis.wns.DoNotificationDocument;

import org.n52.wns.WNSException;
import org.n52.wns.WNSServiceException;
import org.n52.wns.mail.MailHandler;
import org.x52North.wns.WNSConfigDocument;

/**
 * @author Dennis Dahlmann
 * @version 1.0
 */
public class PhoneHandlerEcall implements PhoneHandler {

	private Logger log = Logger.getLogger(PhoneHandlerEcall.class.getName());

	// Ecall's pop server
	private String host;

	private MailHandler mh;

	private String wns;

	/**
	 * constructor PhoneHandlerEcall loads the MailHandler
	 * 
	 * @throws WNSException
	 */
	public PhoneHandlerEcall() throws WNSException {
		mh = org.n52.wns.mail.MailHandlerFactory.getInstance();
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see org.n52.communication.PhoneHandler#setProperties(org.jdom.Element)
	 */
	public void setProperties(
			WNSConfigDocument.WNSConfig.RegisteredHandlers.PhoneHandler phoneHandler,
			String wns) throws WNSServiceException {

		host = phoneHandler.getHost();
		this.wns = wns;

	}

	/**
	 * sends a PHONE NotificationMessage
	 * 
	 * @param user
	 *            The WNSUser
	 * @param message
	 *            The WNSMessage to send
	 */
	public void sendNotificationMessage(DoNotificationDocument dnd,
			String[] targets, String messageID) throws WNSServiceException {

		String subject;
		String body;
		for (int i = 0; i <= targets.length - 1; i++) {
			String msisdn = targets[i].toString();

			// verify format
			msisdn = PhoneNumber.convertToInternationalFormat(msisdn);

			// concat recipient <number>@host

			String to = msisdn + "@" + host;

			subject = ("Web Notification Service");

			body = "Notification Message"
					+ " MessageID: "
					+ messageID
					+ " WNSURL: "
					+ wns
					+ " ServiceURL: "
					+ dnd.getDoNotification().getMessage()
							.getServiceDescription().getServiceURL()
					+ " Message:"
					+ dnd.getDoNotification().getMessage().getShortMessage();

			mh.sendExternalMessage(to, subject, body);

		}
	}

	/**
	 * sends a PHONE Communication Message
	 * 
	 * @param user
	 *            The WNSUser
	 * @param message
	 *            The WNSMessage to send
	 * @throws WNSException
	 */
	public void sendCommunicationMessage(DoCommunicationDocument dcd,
			String[] targets, String messageID) throws WNSServiceException {

		String subject;
		String body;
		for (int i = 0; i <= targets.length - 1; i++) {
			String msisdn = targets[i].toString();

			// verify format
			msisdn = PhoneNumber.convertToInternationalFormat(msisdn);

			// concat recipient <number>@host

			String to = msisdn + "@" + host;

			subject = "Web Notification Service";

			body = "Communication Message"
					+ " MessageID: "
					+ messageID
					+ " WNSURL: "
					+ wns
					+ " ServiceURL: "
					+ dcd.getDoCommunication().getMessage()
							.getServiceDescription().getServiceURL()
					+ " Message: "
					+ dcd.getDoCommunication().getMessage().getShortMessage();
			mh.sendExternalMessage(to, subject, body);

		}
	}
}
