/******************************************************************************
 Web Notification Service (WNS): An open interface for a service
 by which a client may conduct asynchronous dialogues
 (message interchanges) with one or more other services.
 ********************************************************************************/
package org.n52.wns.db;

import java.io.File;
import java.util.Properties;
import java.util.logging.Level;
import java.util.logging.Logger;

import org.n52.wns.WNSException;
import org.n52.wns.WNSInitParamContainer;

/**
 * @author Johannes Echterhoff
 * @version 1.0
 * 
 */
public class XMLFileDAOFactory implements DAOFactory {

	private Logger logger = Logger.getLogger(XMLFileDAOFactory.class.getName());

	private File wnsuserFile = null;

	public XMLFileDAOFactory() {
		// initialise Factory
		WNSInitParamContainer initParams = WNSInitParamContainer.getInstance();
		try {
			this.configureFactory(initParams.getWNSDAOFactoryProperties());
		} catch (Exception e) {
			logger.log(Level.SEVERE, "Could not configure factory.", e);
			// TODO correct handling??
		}
	}

	private void configureFactory(Properties factoryProperties)
			throws WNSException {

		String wnsusersFileLocation = factoryProperties
				.getProperty("wns.user.filelocation");
		if (wnsusersFileLocation == null) {
			throw new WNSException("Required property 'wns.user.filelocation'"
					+ " not found in DAOFactory properties file.");
		}

		this.wnsuserFile = new File(wnsusersFileLocation);
		// test: make sure that the File is a file, that it exists and that the
		// application can read and write it
		if (!wnsuserFile.exists() || !wnsuserFile.isFile()
				|| !wnsuserFile.canRead() || !wnsuserFile.canWrite()) {
			throw new WNSException("The WNSUser file must be accessible, i.e."
					+ " it must exist, be a file and can be read and write by"
					+ " the application.");
		}

		// test: try to retrieve userfile via DAO (throws WNSException if the
		// WNSUserDocument is not valid)
		UserDAO userDao = new XMLFileUserDAO(wnsuserFile);
		userDao.getUserDocument();

		// TODO test MessageDAO
	}

	/**
	 * @see org.n52.wns.db.DAOFactory#getUserDAO()
	 */
	public UserDAO getUserDAO() {
		return new XMLFileUserDAO(wnsuserFile);
	}
}
