/******************************************************************************
 Web Notification Service (WNS): An open interface for a service
 by which a client may conduct asynchronous dialogues
 (message interchanges) with one or more other services.
 *******************************************************************************/
package org.n52.wns.db;

import java.io.File;
import java.io.IOException;
import java.util.ArrayList;
import java.util.Iterator;
import java.util.logging.Level;
import java.util.logging.Logger;

import org.apache.xmlbeans.XmlOptions;
import org.n52.wns.WNSException;
import org.x52North.wns.WNSUserDocument;

/**
 * Stores the WNSUser document in a file. Read and Write accesses are
 * synchronized.
 * 
 * @author Johannes Echterhoff
 * @version 1.0
 */
public class XMLFileUserDAO implements UserDAO {

	private Logger logger = Logger.getLogger(XMLFileUserDAO.class.getName());

	private File wnsuserFile;

	public XMLFileUserDAO(File wnsuserFile) {
		this.wnsuserFile = wnsuserFile;
	}

	/**
	 * @see org.n52.wns.db.UserDAO#getUserDocument()
	 */
	public synchronized WNSUserDocument getUserDocument() throws WNSException {

		try {
			WNSUserDocument userdoc = WNSUserDocument.Factory
					.parse(wnsuserFile);

			// validate WNSUserDocument

			// Set up the validation error listener.
			ArrayList validationErrors = new ArrayList();
			XmlOptions validationOptions = new XmlOptions();
			validationOptions.setErrorListener(validationErrors);

			boolean isValid = userdoc.validate(validationOptions);

			// create XmlException with error-message if the XML is invalid.
			if (!isValid) {
				StringBuffer sb = new StringBuffer();
				Iterator iter = validationErrors.iterator();
				while (iter.hasNext()) {
					sb.append("[Validation-Error] " + iter.next() + "\n");
				}
				throw new WNSException("The WNSUserDocument is invalid: \n"
						+ sb.toString());
			} else {
				logger.info("WNSUserDocument is valid.");
			}

			return userdoc;

		} catch (Exception e) {
			String message = "Could not load WNSUser document.";
			logger.log(Level.SEVERE, message, e);
			throw new WNSException(message, e);
		}
	}

	/**
	 * @see org.n52.wns.db.UserDAO#storeWNSUserDocument(org.x52North.wns.WNSUserDocument)
	 */
	public synchronized void storeWNSUserDocument(WNSUserDocument wnsud)
			throws WNSException {

		// TODO maybe a backup mechanism is necessary, maybe not

		if (wnsud != null) {

			// save WNSUserDocument
			XmlOptions options = new XmlOptions();
			//			
			options.setSavePrettyPrint();
			options.setUseDefaultNamespace();
			options.setSaveAggressiveNamespaces();
			//			
			try {
				wnsud.save(this.wnsuserFile, options);
			} catch (IOException e) {
				String message = "Could not store WNSUser document.";
				logger.log(Level.WARNING, message, e);
				throw new WNSException(message, e);
			}
		}
	}
}
