/******************************************************************************
 Web Notification Service (WNS): An open interface for a service
 by which a client may conduct asynchronous dialogues
 (message interchanges) with one or more other services.
 *******************************************************************************/

package org.n52.wns.mail;

import java.util.logging.Logger;

import net.opengis.wns.CommunicationMessageDocument;
import net.opengis.wns.DoCommunicationDocument;
import net.opengis.wns.DoNotificationDocument;
import net.opengis.wns.NotificationMessageDocument;

import org.apache.commons.mail.EmailException;
import org.apache.commons.mail.SimpleEmail;
import org.n52.wns.WNSServiceException;
import org.n52.wns.WNSServiceException.ExceptionCode;
import org.n52.wns.WNSServiceException.ExceptionLevel;
import org.x52North.wns.WNSConfigDocument;

/**
 * @author Dennis Dahlmann
 * @version 1.0
 */

public class MailHandler {

	private String smtp, smtpuser, smtppwd;

	private String sender;

	SimpleEmail mimeMessage = null;

	private static Logger log = Logger.getLogger(MailHandler.class.getName());

	/** constuctor */
	protected MailHandler(
			WNSConfigDocument.WNSConfig.RegisteredHandlers.EMailHandler mailhandler) {
		setProperties(mailhandler);
	}

	/**
	 * Sends a communication message
	 * 
	 * @param user
	 *            the user to be sent to
	 * @param m
	 *            the message to be sent
	 */
	public void sendCommunicationMessage(DoCommunicationDocument dcd,
			String[] targets) throws WNSServiceException {

		for (int i = 0; i <= targets.length - 1; i++) {
			String recipient = targets[i];
			try {
				mimeMessage = new SimpleEmail();
				mimeMessage.setHostName(smtp);
				mimeMessage.setFrom(sender);
				mimeMessage.addTo(recipient);
				mimeMessage.setSubject("Web Notification Service");
				// write all messages into the message_text area
				CommunicationMessageDocument comdoc = CommunicationMessageDocument.Factory
						.newInstance();
				comdoc.setCommunicationMessage(dcd.getDoCommunication()
						.getMessage());
				mimeMessage.setMsg(comdoc.toString());
				// Send message
				mimeMessage.send();
			} catch (Exception ex) {
				WNSServiceException service = new WNSServiceException(
						ExceptionLevel.PlainExceptions);
				service.addCodedException(ExceptionCode.EmailHandlerError,
						"MailHandler, sendCommunicationMessage");
				throw service;
			}
		}

	}

	/**
	 * Sends a notification email
	 * 
	 * @param user
	 *            TransferObject that holds the user information
	 * @param messageType
	 *            For messageTypes, see OGC-WNS DIPR
	 */
	public void sendNotificationMessage(DoNotificationDocument dnd,
			String[] targets) throws WNSServiceException {

		for (int i = 0; i < targets.length; i++) {
			String recipient = targets[i];
			try {
				mimeMessage = new SimpleEmail();
				mimeMessage.setHostName(smtp);
				mimeMessage.setFrom(sender);
				mimeMessage.addTo(recipient);
				mimeMessage.setSubject("Web Notification Service");
				// write all messages into the message_text area
				NotificationMessageDocument notdoc = NotificationMessageDocument.Factory
						.newInstance();
				notdoc.setNotificationMessage(dnd.getDoNotification()
						.getMessage());
				mimeMessage.setMsg(notdoc.toString());
				// Send message
				mimeMessage.send();

			} catch (Exception ex) {

				System.out.println(ex.getLocalizedMessage());
				WNSServiceException service = new WNSServiceException(
						ExceptionLevel.PlainExceptions);
				service.addCodedException(ExceptionCode.EmailHandlerError,
						"MailHandler, sendNotificationMessage");
				throw service;
			}
		}

	}

	public void sendExternalMessage(String target, String subject, String body)
			throws WNSServiceException {
		try {
			mimeMessage = new SimpleEmail();
			mimeMessage.setHostName(smtp);
			mimeMessage.setFrom(sender);
			mimeMessage.addTo(target);
			mimeMessage.setSubject(subject);
			mimeMessage.setMsg(body);
			mimeMessage.send();
		} catch (EmailException e) {
			WNSServiceException service = new WNSServiceException(
					ExceptionLevel.PlainExceptions);
			service
					.addCodedException(ExceptionCode.EmailHandlerError,
							"MailHandler, sendExternalMessage", e
									.getLocalizedMessage());
			throw service;
		}
	}

	/** sets the mail session properties at startup */
	private void setProperties(
			WNSConfigDocument.WNSConfig.RegisteredHandlers.EMailHandler mailhandler) {
		smtp = mailhandler.getSMTP().getHost();
		smtppwd = mailhandler.getSMTP().getPasswd();
		smtpuser = mailhandler.getSMTP().getUser();
		sender = mailhandler.getSMTP().getSender();

	}
}
