/******************************************************************************
 Web Notification Service (WNS): An open interface for a service
 by which a client may conduct asynchronous dialogues
 (message interchanges) with one or more other services.
 *******************************************************************************/

package org.n52.wns.mail;

import org.n52.wns.WNSException;

/**
 * 
 * @author Dennis Dahlmann
 * @version 1.0
 */
public class MailHandlerFactory {
	private static MailHandler instance = null;

	/**
	 * Get method for singelton MailHandler
	 * 
	 * @return The MailHandler
	 * @throws WNSException
	 */
	public static MailHandler getInstance() throws WNSException {
		if (instance == null)
			throw new WNSException(
					"MailHandlerFactory: Cannot instanciate without configuration data!");
		else
			return instance;
	}

	/**
	 * Get method for singleton MailHandler
	 * 
	 * @return MailHandler
	 */
	public static MailHandler getInstance(
			org.x52North.wns.WNSConfigDocument.WNSConfig.RegisteredHandlers.EMailHandler mailHandler) {
		if (instance == null) {
			synchronized (org.n52.wns.mail.MailHandlerFactory.class) {
				if (instance == null) {
					instance = new org.n52.wns.mail.MailHandler(mailHandler);
				}
			}
		}
		return instance;
	}
}
