/******************************************************************************
 Web Notification Service (WNS): An open interface for a service
 by which a client may conduct asynchronous dialogues
 (message interchanges) with one or more other services.
 *******************************************************************************/

package org.n52.wns.sms;

import java.util.logging.Logger;

import net.opengis.wns.DoCommunicationDocument;
import net.opengis.wns.DoNotificationDocument;

import org.n52.wns.WNSException;
import org.n52.wns.WNSServiceException;
import org.n52.wns.communication.PhoneNumber;
import org.n52.wns.mail.MailHandler;

/**
 * Specific SMSHandler for the Ecall-Account http://www.ecall.com Ecall uses an
 * email-to-sms/speech/fax mechanism. Therefore SMS are sent as emails.
 * 
 * @author Dennis Dahlmann
 * @version 1.0S
 */
public class SMSHandlerEcall implements SMSHandler {

	private static Logger log = Logger.getLogger(SMSHandlerEcall.class
			.getName());

	private MailHandler mh = null;

	private String mailtoAdress;

	private String wnsURL;

	/**
	 * SMSHandlerEcall loads system properties to set up the mail environment
	 * 
	 * @throws WNSException
	 */
	public SMSHandlerEcall() throws WNSException {
		// loads the MailHandler used to send mails to ecall
		mh = org.n52.wns.mail.MailHandlerFactory.getInstance();
	}

	/**
	 * @see org.n52.wns.sms.SMSHandler#setProperties(org.jdom.Element)
	 */
	public void setProperties(
			org.x52North.wns.WNSConfigDocument.WNSConfig.RegisteredHandlers.SMSHandler shd,
			String wns) throws WNSServiceException {

		mailtoAdress = shd.getHost();
		this.wnsURL = wns;
	}

	/**
	 * sends a SMS NotificationMessage
	 * 
	 * @param user
	 * @param message
	 */
	public void sendNotificationMessage(DoNotificationDocument dnd,
			String[] targets, String messageID) throws WNSServiceException {

		String text = "Notification, MessageID:"
				+ messageID
				+ " WNSURL: "
				+ wnsURL
				+ " ServiceURL: "
				+ dnd.getDoNotification().getMessage().getServiceDescription()
						.getServiceURL() + " Message: "
				+ dnd.getDoNotification().getMessage().getShortMessage();
		for (int i = 0; i <= targets.length - 1; i++) {
			String msisdn = targets[i].toString();
			// verify format and set subject
			String to = PhoneNumber.convertToInternationalFormat(msisdn) + "@"
					+ mailtoAdress;
			mh.sendExternalMessage(to, "", text);
			System.out.println("SMS sending successful");
		}

	}

	/**
	 * sends a SMS Communication Message
	 * 
	 * @param user
	 * @param message
	 * @throws WNSException
	 */
	public void sendCommunicationMessage(DoCommunicationDocument dcd,
			String[] targets, String messageID) throws WNSServiceException {

		String text = "Communication, MessageID:"
				+ messageID
				+ " WNSURL: "
				+ wnsURL
				+ " ServiceURL: "
				+ dcd.getDoCommunication().getMessage().getServiceDescription()
						.getServiceURL() + "Message: "
				+ dcd.getDoCommunication().getMessage().getShortMessage();

		for (int i = 0; i <= targets.length - 1; i++) {
			String msisdn = targets[i].toString();
			// verify format and set subject
			String to = PhoneNumber.convertToInternationalFormat(msisdn) + "@"
					+ mailtoAdress;

			mh.sendExternalMessage(to, "", text);
		}

	}

}
